/*
 /*++
    Copyright (c) 2008-2009 SMSC. All rights reserved.

    The information contained in this file is confidential and proprietary to
    Standard Microsystems Corporation.  No part of this file may be reproduced
    or distributed, in any form or by any means for any purpose, without the
    express written permission of Standard Microsystems Corporation.
Module Name:
    Ioctl.c
Abstract:
    This module contains all the IoCtl funtions
Revision History:
    02/17/2009  Initial version.
Notes:
--*/

#include "smsc9500.h"

NIC_STATUS DumpLanRegs (PMP_ADAPTER Adapter)
{
	UINT    i;
	DWORD Value32;
	NIC_STATUS  Status=NIC_STATUS_SUCCESS;

	RETAILMSG(1, (TEXT("Dump Lan9500 Registers\r\n")));
		
	for (i=0U;i<(sizeof(sysCsr)/sizeof(SHOW_REG));i++) {		
		Status=NICReadRegister(Adapter, sysCsr[i].dwOffset, &Value32);
		if (Status != NIC_STATUS_SUCCESS) {
			SMSC_TRACE1(DBG_ERROR, "Read register failed in DumpLanRegs(), Status= 0x%x \r\n",Status);
			return Status;
		}
		RETAILMSG(1, (TEXT("%-20s = 0x%08x\r\n"), sysCsr[i].wszName, Value32));
	}
	return Status;
}

NIC_STATUS DumpMacRegs (PMP_ADAPTER Adapter)
{
	UINT    i;
   	DWORD Value32;
	NIC_STATUS  Status=NIC_STATUS_SUCCESS;

	RETAILMSG(1, (TEXT("Dump Lan9500 Mac Registers\r\n")));

	for (i=0U;i<(sizeof(macCsr)/sizeof(SHOW_REG));i++) {
	Status=NICReadRegister(Adapter, macCsr[i].dwOffset, &Value32);
	if (Status != NIC_STATUS_SUCCESS) {
			SMSC_TRACE1(DBG_ERROR, "Read register failed in DumpMacRegs(), Status= 0x%x \r\n",Status);
			return Status;
		}
		RETAILMSG(1, (TEXT("%-20s = 0x%08x\r\n"), macCsr[i].wszName, Value32));
		}
	return Status;
}

NIC_STATUS DumpPhyRegs (PMP_ADAPTER Adapter)
{
	UINT    i;
	DWORD Value32;
	NIC_STATUS  Status=NIC_STATUS_SUCCESS;

	if (Adapter->MdioDisable & MDIO_DISABLE_INTERNAL) {
		return NIC_STATUS_FAILURE;
	}
	RETAILMSG(1, (TEXT("Dump Lan9500 Phy Registers\r\n")));
		
	for (i=0U;i<(sizeof(phyCsr)/sizeof(SHOW_REG));i++) {
		Status=NICReadPhy(Adapter, phyCsr[i].dwOffset, &Value32);
		if (Status != NIC_STATUS_SUCCESS) {
			SMSC_TRACE1(DBG_ERROR, "Read register failed in DumpPHYRegs(), Status= 0x%x \r\n",Status);
			return Status;
		}
		RETAILMSG(1, (TEXT("%-20s = 0x%08x\r\n"), phyCsr[i].wszName, Value32));
	}
	return Status;
}

void DumpRXStatus (PMP_ADAPTER Adapter)
{ 		
	RETAILMSG(1, (TEXT("Dump RX Status\r\n")));
	RETAILMSG(1,(TEXT("RxGoodFrames			= %d\r\n"), Adapter->nPacketsIndicatedOnRx));
	RETAILMSG(1,(TEXT("RxCrcErrors			= %d\r\n"), Adapter->RcvCrcErrors));
	RETAILMSG(1,(TEXT("RxRuntFrameErrors	= %d\r\n"), Adapter->RcvRuntErrors));
	RETAILMSG(1,(TEXT("RxAlignmentErrors	= %d\r\n"), Adapter->RcvAlignmentErrors));
	RETAILMSG(1,(TEXT("RxFrameTooLongError	= %d\r\n"), Adapter->RcvFrameTooLongErrors));
	RETAILMSG(1,(TEXT("RxLaterCollisionError= %d\r\n"), Adapter->RcvLateCollisionErrors));
	RETAILMSG(1,(TEXT("RxBadFrames			= %d\r\n"), Adapter->RcvBadErrors));
	RETAILMSG(1,(TEXT("RxFifoDroppedFrames	= %d\r\n"), Adapter->RcvResourceErrors));
	RETAILMSG(1,(TEXT("EeeRxLpiTransitions	= %d\n"), Adapter->EeeRxLpiTransitions));
	RETAILMSG(1,(TEXT("EeeRxLpiTime			= %d\n"), Adapter->EeeRxLpiTime));	   
}

void  DumpTXStatus (PMP_ADAPTER Adapter)
{ 	
	RETAILMSG(1, (TEXT("Dump TX Status\r\n")));
	RETAILMSG(1,(TEXT("TxGoodFrames					= %d\r\n"), Adapter->nPacketsTransmittedSuccessfully));
	RETAILMSG(1,(TEXT("TxPauseFrames				= %d\r\n"), Adapter->TxPauseFrames));
	RETAILMSG(1,(TEXT("TxSingleCollisions			= %d\r\n"), Adapter->OneRetry));
	RETAILMSG(1,(TEXT("TxMultipleCollisions			= %d\r\n"), Adapter->MoreThanOneRetry));
	RETAILMSG(1,(TEXT("TxExcessiveCollisionErrors	= %d\r\n"), Adapter->TxAbortExcessCollisions));
	RETAILMSG(1,(TEXT("TxLateCollisionErrors		= %d\r\n"), Adapter->TxLateCollisions));
	RETAILMSG(1,(TEXT("TxBufferUnderrunErrors		= %d\r\n"), Adapter->TxDmaUnderrun));
	RETAILMSG(1,(TEXT("TxExcessiveDeferralErrors	= %d\r\n"), Adapter->TransmitFailuresOther));
	RETAILMSG(1,(TEXT("EeeTxLpiTransitions			= %d\n"), Adapter->EeeTxLpiTransitions));
	RETAILMSG(1,(TEXT("EeeTxLpiTime					= %d\n"), Adapter->EeeTxLpiTime));
}

BOOL GetE2pSize (PMP_ADAPTER Adapter, int *E2pSize)
{
	DWORD dwTemp, dwValue;
	BOOL IsTheraChipId=FALSE;

	// Read ETH_STRAP_DEBUG
	if (NICReadRegister(Adapter, ETH_ID_REV , &dwValue) == NIC_STATUS_FAILURE) {
        	return FALSE;
	}
	if ((dwValue&0xFF000000)==0x95000000) {
		IsTheraChipId=TRUE;
	}
	
	if (IsTheraChipId) {
		// Read ETH_STRAP_DEBUG
		if (NICReadRegister(Adapter, ETH_STRAP_DEBUG, &dwTemp) == NIC_STATUS_FAILURE) {
			SMSC_TRACE0(DBG_ERROR,"GetE2pSize(), Failed to read register ETH_STRAP_DEBUG \r\n");
			return FALSE;
		}
		// Check EEPROM Disable strap
		if (dwTemp & 0x1UL) {
			RETAILMSG(1,(TEXT("WARNING::EEPROM Disable Strap is set!\r\n")));
			return FALSE;
		}
	}

	// Check EEPROM SIZE strap value
	if (IsTheraChipId && (!(dwTemp & 0x4UL))) {// 128-byte eeprom
		*E2pSize = 128;
		return TRUE;
	} else {    	   	 			
        // Check to see if it is 256-byte E2P
		BYTE bStart, bEnd, bTemp;

		if (NICReadEepromByte(Adapter, 0, &bStart) == NIC_STATUS_FAILURE) {
			SMSC_TRACE0(DBG_ERROR,"GetE2pSize(), Failed NICReadEepromByte() at 0\r\n");
			return FALSE;
		}
	 
		if (NICReadEepromByte(Adapter, 256, &bEnd) == NIC_STATUS_FAILURE) {
			SMSC_TRACE0(DBG_ERROR,"GetE2pSize(), Failed at NICReadEepromByte()  at 256\r\n");
			return FALSE;
		}
	
		// Write to 256 the inverse value of bStart
		bTemp = ~bStart;
		if (NICWriteEepromByte(Adapter, 256, &bTemp) == NIC_STATUS_FAILURE) {
			SMSC_TRACE0(DBG_ERROR,"GetE2pSize(), Failed at NICWriteEepromByte()  at 256\r\n");
			return FALSE;
		}
        
		// Now read location zero again
		if (NICReadEepromByte(Adapter, 0, &bTemp) == NIC_STATUS_FAILURE) {
			SMSC_TRACE0(DBG_ERROR,"GetE2pSize(), Failed at NICReadEepromByte() at 0\r\n");
			return FALSE;
		}

		if (bTemp != bStart) {// 256-byte E2P
			// Rewrite correct value into location 0
			if (NICWriteEepromByte(Adapter, 0, &bStart) == NIC_STATUS_FAILURE) {
				SMSC_TRACE0(DBG_ERROR,"GetE2pSize(), Failed at NICWriteEepromByte() at 0\r\n");
				return FALSE;
			}
			*E2pSize = 256;
			return TRUE;
		} else { // 512-byte E2P
			// Rewrite correct value into location 256
			if (NICWriteEepromByte(Adapter, 256, &bEnd) == NIC_STATUS_FAILURE) {
				SMSC_TRACE0(DBG_ERROR,"GetE2pSize(), Failed at NICWriteEepromByte() at 256\r\n");
				return FALSE;
			}

			*E2pSize = 512;
			return TRUE;
		}
	}    
}

void
DumpEepRAM(
	IN PMP_ADAPTER Adapter
    )
/*++
Routine Description:
	Prints out contents of EEP RAM 
Arguments:
	Adapter    Pointer to our adapter
Return Value:
	None
--*/    
{
    DWORD i;
    DWORD Temp32,Value32;

    Temp32=0xDEADBEEF;
    NICWriteRegister( Adapter, ETH_DP_DATA0, &Temp32);

    // Enable EEP RAM access
    Temp32=ETH_DP_SEL_TX_EEP_RAM_ | ETH_DP_SEL_TESTEN_;
    NICWriteRegister( Adapter, ETH_DP_SEL, &Temp32);
	for (i=0;i<32;i++) {
        EepRamRead(Adapter, i, &Value32);
        SMSC_TRACE2(DBG_EEPROM,"EEP[%02d] = 0x%08lx\r\n", i, Value32);
    }

    // Shutoff EEP RAM access
    Temp32=0;
    NICWriteRegister( Adapter, ETH_DP_SEL, 0);
    Temp32=0xBEEFDEAD;
    NICWriteRegister( Adapter, ETH_DP_DATA0, &Temp32);
}



