/*++

	Copyright (c) 2008-2009 SMSC. All rights reserved.

	The information contained in this file is confidential and proprietary to
	Standard Microsystems Corporation.	No part of this file may be reproduced
	or distributed, in any form or by any means for any purpose, without the
	express written permission of Standard Microsystems Corporation.


Module Name:

	Request.c

Abstract:

		This module contains Miniport function and helper routines for handling
		Set & Query Information requests.

Revision History:
	12/11/2008	Initial version.

Notes:

--*/


#include "smsc9500.h"

NDIS_OID NICSupportedOidsAll[] =
{
		OID_GEN_SUPPORTED_LIST,
		OID_GEN_HARDWARE_STATUS,
		OID_GEN_MEDIA_SUPPORTED,
		OID_GEN_MEDIA_IN_USE,
		OID_GEN_MAXIMUM_LOOKAHEAD,
		OID_GEN_MAXIMUM_FRAME_SIZE,
		OID_GEN_LINK_SPEED,
		OID_GEN_TRANSMIT_BUFFER_SPACE,
		OID_GEN_RECEIVE_BUFFER_SPACE,
		OID_GEN_TRANSMIT_BLOCK_SIZE,
		OID_GEN_RECEIVE_BLOCK_SIZE,
		OID_GEN_VENDOR_ID,
		OID_GEN_VENDOR_DESCRIPTION,
		OID_GEN_VENDOR_DRIVER_VERSION,
		OID_GEN_CURRENT_PACKET_FILTER,
		OID_GEN_CURRENT_LOOKAHEAD,
		OID_GEN_DRIVER_VERSION,
		OID_GEN_MAXIMUM_TOTAL_SIZE,
		OID_GEN_MAC_OPTIONS,
		OID_GEN_MEDIA_CONNECT_STATUS,
		OID_GEN_MAXIMUM_SEND_PACKETS,
		OID_GEN_XMIT_OK,
		OID_GEN_RCV_OK,
		OID_GEN_XMIT_ERROR,
		OID_GEN_RCV_ERROR,
		OID_GEN_RCV_NO_BUFFER,
		OID_GEN_RCV_CRC_ERROR,
		OID_GEN_TRANSMIT_QUEUE_LENGTH,
		OID_802_3_PERMANENT_ADDRESS,
		OID_802_3_CURRENT_ADDRESS,
		OID_802_3_MULTICAST_LIST,
		OID_802_3_MAC_OPTIONS,
		OID_802_3_MAXIMUM_LIST_SIZE,
		OID_802_3_RCV_ERROR_ALIGNMENT,
		OID_802_3_XMIT_ONE_COLLISION,
		OID_802_3_XMIT_MORE_COLLISIONS,
		OID_802_3_XMIT_DEFERRED,
		OID_802_3_XMIT_MAX_COLLISIONS,
		OID_802_3_RCV_OVERRUN,
		OID_802_3_XMIT_UNDERRUN,
		OID_802_3_XMIT_HEARTBEAT_FAILURE,
		OID_802_3_XMIT_TIMES_CRS_LOST,
		OID_802_3_XMIT_LATE_COLLISIONS,
		OID_PNP_CAPABILITIES,
		OID_PNP_SET_POWER,
		OID_PNP_QUERY_POWER,
		OID_PNP_ADD_WAKE_UP_PATTERN,
		OID_PNP_REMOVE_WAKE_UP_PATTERN,
		OID_PNP_ENABLE_WAKE_UP
};

NDIS_OID NICSupportedOidsNoPatternWake[] =
{
		OID_GEN_SUPPORTED_LIST,
		OID_GEN_HARDWARE_STATUS,
		OID_GEN_MEDIA_SUPPORTED,
		OID_GEN_MEDIA_IN_USE,
		OID_GEN_MAXIMUM_LOOKAHEAD,
		OID_GEN_MAXIMUM_FRAME_SIZE,
		OID_GEN_LINK_SPEED,
		OID_GEN_TRANSMIT_BUFFER_SPACE,
		OID_GEN_RECEIVE_BUFFER_SPACE,
		OID_GEN_TRANSMIT_BLOCK_SIZE,
		OID_GEN_RECEIVE_BLOCK_SIZE,
		OID_GEN_VENDOR_ID,
		OID_GEN_VENDOR_DESCRIPTION,
		OID_GEN_VENDOR_DRIVER_VERSION,
		OID_GEN_CURRENT_PACKET_FILTER,
		OID_GEN_CURRENT_LOOKAHEAD,
		OID_GEN_DRIVER_VERSION,
		OID_GEN_MAXIMUM_TOTAL_SIZE,
		OID_GEN_MAC_OPTIONS,
		OID_GEN_MEDIA_CONNECT_STATUS,
		OID_GEN_MAXIMUM_SEND_PACKETS,
		OID_GEN_XMIT_OK,
		OID_GEN_RCV_OK,
		OID_GEN_XMIT_ERROR,
		OID_GEN_RCV_ERROR,
		OID_GEN_RCV_NO_BUFFER,
		OID_GEN_RCV_CRC_ERROR,
		OID_GEN_TRANSMIT_QUEUE_LENGTH,
		OID_802_3_PERMANENT_ADDRESS,
		OID_802_3_CURRENT_ADDRESS,
		OID_802_3_MULTICAST_LIST,
		OID_802_3_MAC_OPTIONS,
		OID_802_3_MAXIMUM_LIST_SIZE,
		OID_802_3_RCV_ERROR_ALIGNMENT,
		OID_802_3_XMIT_ONE_COLLISION,
		OID_802_3_XMIT_MORE_COLLISIONS,
		OID_802_3_XMIT_DEFERRED,
		OID_802_3_XMIT_MAX_COLLISIONS,
		OID_802_3_RCV_OVERRUN,
		OID_802_3_XMIT_UNDERRUN,
		OID_802_3_XMIT_HEARTBEAT_FAILURE,
		OID_802_3_XMIT_TIMES_CRS_LOST,
		OID_802_3_XMIT_LATE_COLLISIONS,
		OID_PNP_CAPABILITIES,
		OID_PNP_SET_POWER,
		OID_PNP_QUERY_POWER,
		OID_PNP_ENABLE_WAKE_UP
};

NDIS_OID NICSupportedOidsNoWakes[] =
{
		OID_GEN_SUPPORTED_LIST,
		OID_GEN_HARDWARE_STATUS,
		OID_GEN_MEDIA_SUPPORTED,
		OID_GEN_MEDIA_IN_USE,
		OID_GEN_MAXIMUM_LOOKAHEAD,
		OID_GEN_MAXIMUM_FRAME_SIZE,
		OID_GEN_LINK_SPEED,
		OID_GEN_TRANSMIT_BUFFER_SPACE,
		OID_GEN_RECEIVE_BUFFER_SPACE,
		OID_GEN_TRANSMIT_BLOCK_SIZE,
		OID_GEN_RECEIVE_BLOCK_SIZE,
		OID_GEN_VENDOR_ID,
		OID_GEN_VENDOR_DESCRIPTION,
		OID_GEN_VENDOR_DRIVER_VERSION,
		OID_GEN_CURRENT_PACKET_FILTER,
		OID_GEN_CURRENT_LOOKAHEAD,
		OID_GEN_DRIVER_VERSION,
		OID_GEN_MAXIMUM_TOTAL_SIZE,
		OID_GEN_MAC_OPTIONS,
		OID_GEN_MEDIA_CONNECT_STATUS,
		OID_GEN_MAXIMUM_SEND_PACKETS,
		OID_GEN_XMIT_OK,
		OID_GEN_RCV_OK,
		OID_GEN_XMIT_ERROR,
		OID_GEN_RCV_ERROR,
		OID_GEN_RCV_NO_BUFFER,
		OID_GEN_RCV_CRC_ERROR,
		OID_GEN_TRANSMIT_QUEUE_LENGTH,
		OID_802_3_PERMANENT_ADDRESS,
		OID_802_3_CURRENT_ADDRESS,
		OID_802_3_MULTICAST_LIST,
		OID_802_3_MAC_OPTIONS,
		OID_802_3_MAXIMUM_LIST_SIZE,
		OID_802_3_RCV_ERROR_ALIGNMENT,
		OID_802_3_XMIT_ONE_COLLISION,
		OID_802_3_XMIT_MORE_COLLISIONS,
		OID_802_3_XMIT_DEFERRED,
		OID_802_3_XMIT_MAX_COLLISIONS,
		OID_802_3_RCV_OVERRUN,
		OID_802_3_XMIT_UNDERRUN,
		OID_802_3_XMIT_HEARTBEAT_FAILURE,
		OID_802_3_XMIT_TIMES_CRS_LOST,
		OID_802_3_XMIT_LATE_COLLISIONS,
		OID_PNP_CAPABILITIES,
		OID_PNP_SET_POWER,
		OID_PNP_QUERY_POWER
};

NDIS_OID NICSupportedOidsNoPM[] =
{
		OID_GEN_SUPPORTED_LIST,
		OID_GEN_HARDWARE_STATUS,
		OID_GEN_MEDIA_SUPPORTED,
		OID_GEN_MEDIA_IN_USE,
		OID_GEN_MAXIMUM_LOOKAHEAD,
		OID_GEN_MAXIMUM_FRAME_SIZE,
		OID_GEN_LINK_SPEED,
		OID_GEN_TRANSMIT_BUFFER_SPACE,
		OID_GEN_RECEIVE_BUFFER_SPACE,
		OID_GEN_TRANSMIT_BLOCK_SIZE,
		OID_GEN_RECEIVE_BLOCK_SIZE,
		OID_GEN_VENDOR_ID,
		OID_GEN_VENDOR_DESCRIPTION,
		OID_GEN_VENDOR_DRIVER_VERSION,
		OID_GEN_CURRENT_PACKET_FILTER,
		OID_GEN_CURRENT_LOOKAHEAD,
		OID_GEN_DRIVER_VERSION,
		OID_GEN_MAXIMUM_TOTAL_SIZE,
		OID_GEN_MAC_OPTIONS,
		OID_GEN_MEDIA_CONNECT_STATUS,
		OID_GEN_MAXIMUM_SEND_PACKETS,
		OID_GEN_XMIT_OK,
		OID_GEN_RCV_OK,
		OID_GEN_XMIT_ERROR,
		OID_GEN_RCV_ERROR,
		OID_GEN_RCV_NO_BUFFER,
		OID_GEN_RCV_CRC_ERROR,
		OID_GEN_TRANSMIT_QUEUE_LENGTH,
		OID_802_3_PERMANENT_ADDRESS,
		OID_802_3_CURRENT_ADDRESS,
		OID_802_3_MULTICAST_LIST,
		OID_802_3_MAC_OPTIONS,
		OID_802_3_MAXIMUM_LIST_SIZE,
		OID_802_3_RCV_ERROR_ALIGNMENT,
		OID_802_3_XMIT_ONE_COLLISION,
		OID_802_3_XMIT_MORE_COLLISIONS,
		OID_802_3_XMIT_DEFERRED,
		OID_802_3_XMIT_MAX_COLLISIONS,
		OID_802_3_RCV_OVERRUN,
		OID_802_3_XMIT_UNDERRUN,
		OID_802_3_XMIT_HEARTBEAT_FAILURE,
		OID_802_3_XMIT_TIMES_CRS_LOST,
		OID_802_3_XMIT_LATE_COLLISIONS
};


/*++

Routine Description:

	Entry point called by NDIS to query for the value of the specified OID.
	

Arguments:

	MiniportAdapterContext			Pointer to the adapter structure
	Oid 									Oid for this query
	InformationBuffer					Buffer for information
	InformationBufferLength 		Size of this buffer
	BytesWritten						Specifies how much info is written
	BytesNeeded 						In case the buffer is smaller than
											what we need, tell them how much is needed

Return Value:

	Return code from the NdisRequest below.

Notes: Read "Minimizing Miniport Driver Initialization Time" in the DDK
	for more info on how to handle certain OIDs that affect the init of
	a miniport.

--*/
NDIS_STATUS
MPQueryInformation(
	IN NDIS_HANDLE	MiniportAdapterContext,
	IN NDIS_OID 	Oid,
	IN PVOID		InformationBuffer,
	IN ULONG		InformationBufferLength,
	OUT PULONG		BytesWritten,
	OUT PULONG		BytesNeeded)
{
	NDIS_STATUS 			Status = NDIS_STATUS_SUCCESS;
	PMP_ADAPTER 			Adapter;
	NDIS_HARDWARE_STATUS	HardwareStatus = NdisHardwareStatusReady;
	NDIS_MEDIUM 			Medium = NIC_MEDIA_TYPE;
	UCHAR					VendorDesc[] = DRIVER_COMPANYNAME_STR;
	ULONG					ulInfo = 0;// index;
	USHORT					usInfo = 0;
	ULONG64 				ulInfo64 = 0;
	PVOID					pInfo = (PVOID) &ulInfo;
	ULONG					ulInfoLen = sizeof(ulInfo);
	NDIS_PNP_CAPABILITIES	PMCaps;

	Adapter = (PMP_ADAPTER) MiniportAdapterContext;

	if(Adapter == NULL){
		SMSC_TRACE0(DBG_ERROR,"MPQueryInformation Adapter = NULL\r\n");
		return NDIS_STATUS_FAILURE;
	}

	NdisAcquireSpinLock(&Adapter->RequestLock);

   SMSC_TRACE1(DBG_REQUEST,"--> MPQueryInformation (0x%08lx)\r\n",	Oid);

	// Initialize the result
	*BytesWritten = 0;
	*BytesNeeded = 0;

	if (MP_TEST_FLAG(Adapter, fMP_SURPRISE_REMOVED))
	{
		Status = NDIS_STATUS_NOT_ACCEPTED;
		goto Exit_MPQueryInformation;
	}

	switch(Oid)
	{
		case OID_GEN_SUPPORTED_LIST:
			//
			// The OID_GEN_SUPPORTED_LIST OID specifies an array of OIDs
			// for objects that the underlying driver or its NIC supports.
			// Objects include general, media-specific, and implementation-
			// specific objects. NDIS forwards a subset of the returned
			// list to protocols that make this query. That is, NDIS filters
			// any supported statistics OIDs out of the list because
			// protocols never make statistics queries.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_SUPPORTED_LIST\r\n");
			if (Adapter->PowerManagement == DISABLED)
			{
				pInfo = (PVOID) NICSupportedOidsNoPM;
				ulInfoLen = sizeof(NICSupportedOidsNoPM);
				SMSC_TRACE0(DBG_REQUEST," NICSupportedOidsNoPM\r\n");
			}
			else if (
						(Adapter->WoLinkChgSettings == DISABLED) &&
						(Adapter->WolMagicPacketSettings == DISABLED) &&
						(Adapter->WolPatternMatchSettings == DISABLED)
					)
			{
				pInfo = (PVOID) NICSupportedOidsNoWakes;
				ulInfoLen = sizeof(NICSupportedOidsNoWakes);

				SMSC_TRACE0(DBG_REQUEST," NICSupportedOidsNoWakes\r\n");
			}
			else if (Adapter->WolPatternMatchSettings == DISABLED)
			{
				pInfo = (PVOID) NICSupportedOidsNoPatternWake;
				ulInfoLen = sizeof(NICSupportedOidsNoPatternWake);

				SMSC_TRACE0(DBG_REQUEST," NICSupportedOidsNoPatternWake\r\n");
			}
			else
			{
				pInfo = (PVOID) NICSupportedOidsAll;
				ulInfoLen = sizeof(NICSupportedOidsAll);

				SMSC_TRACE0(DBG_REQUEST," NICSupportedOidsAll\r\n");
			}
			break;

		case OID_GEN_HARDWARE_STATUS:
			//
			// Specify the current hardware status of the underlying NIC as
			// one of the following NDIS_HARDWARE_STATUS-type values.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_HARDWARE_STATUS\r\n");
			pInfo = (PVOID) &HardwareStatus;
			ulInfoLen = sizeof(NDIS_HARDWARE_STATUS);
			SMSC_TRACE1(DBG_REQUEST,"HardwareStatus = 0x%x \r\n", HardwareStatus);
			break;

		case OID_GEN_MEDIA_SUPPORTED:
			//
			// Specify the media types that the NIC can support but not
			// necessarily the media types that the NIC currently uses.
			// fallthrough:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_MEDIA_SUPPORTED\r\n");
		case OID_GEN_MEDIA_IN_USE:
			//
			// Specifiy a complete list of the media types that the NIC
			// currently uses.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_MEDIA_IN_USE\r\n");
			pInfo = (PVOID) &Medium;
			ulInfoLen = sizeof(NDIS_MEDIUM);
			SMSC_TRACE1(DBG_REQUEST,"Medium = 0x%x \r\n", Medium);
			break;

		case OID_GEN_CURRENT_LOOKAHEAD:
		case OID_GEN_MAXIMUM_LOOKAHEAD:
			//
			// If the miniport driver indicates received data by calling
			// NdisXxxIndicateReceive, it should respond to OID_GEN_MAXIMUM_LOOKAHEAD
			// with the maximum number of bytes the NIC can provide as
			// lookahead data. If that value is different from the size of the
			// lookahead buffer supported by bound protocols, NDIS will call
			// MiniportSetInformation to set the size of the lookahead buffer
			// provided by the miniport driver to the minimum of the miniport
			// driver and protocol(s) values. If the driver always indicates
			// up full packets with NdisMIndicateReceivePacket, it should
			// set this value to the maximum total packet size, which
			// excludes the header.
			// Upper-layer drivers examine lookahead data to determine whether
			// a packet that is associated with the lookahead data is intended
			// for one or more of their clients. If the underlying driver
			// supports multipacket receive indications, bound protocols are
			// given full net packets on every indication. Consequently,
			// this value is identical to that returned for
			// OID_GEN_RECEIVE_BLOCK_SIZE.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_CURRENT_LOOKAHEAD\r\n");
			if(Adapter->ulLookAhead == 0)
			{
				Adapter->ulLookAhead = NIC_MAX_LOOKAHEAD;
			}
			ulInfo = Adapter->ulLookAhead;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo = 0x%x \r\n", ulInfo);
			break;

		case OID_GEN_MAXIMUM_FRAME_SIZE:
			//
			// Specifiy the maximum network packet size, in bytes, that the
			// NIC supports excluding the header. A NIC driver that emulates
			// another medium type for binding to a transport must ensure that
			// the maximum frame size for a protocol-supplied net packet does
			// not exceed the size limitations for the true network medium.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_MAXIMUM_FRAME_SIZE\r\n");
			ulInfo = ETH_MAX_PACKET_SIZE - ETH_HEADER_SIZE;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_MAXIMUM_TOTAL_SIZE:
			//
			// Specify the maximum total packet length, in bytes, the NIC
			// supports including the header. A protocol driver might use
			// this returned length as a gauge to determine the maximum
			// size packet that a NIC driver could forward to the
			// protocol driver. The miniport driver must never indicate
			// up to the bound protocol driver packets received over the
			// network that are longer than the packet size specified by
			// OID_GEN_MAXIMUM_TOTAL_SIZE.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_MAXIMUM_TOTAL_SIZE\r\n");
		case OID_GEN_TRANSMIT_BLOCK_SIZE:
			//
			// The OID_GEN_TRANSMIT_BLOCK_SIZE OID specifies the minimum
			// number of bytes that a single net packet occupies in the
			// transmit buffer space of the NIC. For example, a NIC that
			// has a transmit space divided into 256-byte pieces would have
			// a transmit block size of 256 bytes. To calculate the total
			// transmit buffer space on such a NIC, its driver multiplies
			// the number of transmit buffers on the NIC by its transmit
			// block size. In our case, the transmit block size is
			// identical to its maximum packet size.
			SMSC_TRACE0(DBG_REQUEST,"OID_GEN_TRANSMIT_BLOCK_SIZE\r\n");
		case OID_GEN_RECEIVE_BLOCK_SIZE:
			//
			// The OID_GEN_RECEIVE_BLOCK_SIZE OID specifies the amount of
			// storage, in bytes, that a single packet occupies in the receive
			// buffer space of the NIC.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_RECEIVE_BLOCK_SIZE\r\n");
			ulInfo = (ULONG) ETH_MAX_PACKET_SIZE;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_MAC_OPTIONS:
			//
			// Specify a bitmask that defines optional properties of the NIC.
			// This miniport indicates receive with NdisMIndicateReceivePacket
			// function. It has no MiniportTransferData function. Such a driver
			// should set this NDIS_MAC_OPTION_TRANSFERS_NOT_PEND flag.
			//
			// NDIS_MAC_OPTION_NO_LOOPBACK tells NDIS that NIC has no internal
			// loopback support so NDIS will manage loopbacks on behalf of
			// this driver.
			//
			// NDIS_MAC_OPTION_COPY_LOOKAHEAD_DATA tells the protocol that
			// our receive buffer is not on a device-specific card. If
			// NDIS_MAC_OPTION_COPY_LOOKAHEAD_DATA is not set, multi-buffer
			// indications are copied to a single flat buffer.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_MAC_OPTIONS\r\n");
			ulInfo = NDIS_MAC_OPTION_COPY_LOOKAHEAD_DATA |
				NDIS_MAC_OPTION_TRANSFERS_NOT_PEND |
				NDIS_MAC_OPTION_NO_LOOPBACK;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_LINK_SPEED:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_LINK_SPEED\r\n");
#if 0 // Indicate media connection status immediatly. It fixes the issue of display getting delayed.
			if ((!Adapter->PerformingNWAY && !MP_TEST_FLAG(Adapter, fMP_RESET_IN_PROGRESS)&& !MP_TEST_FLAG(Adapter, fMP_INIT_IN_PROGRESS))
				|| MP_TEST_FLAG(Adapter,fMP_DEVICE_NOT_ATTACHED))
			{
				//
				// Specify the maximum speed of the NIC in kbps.
				//
				ulInfo = NICGetLinkSpeed(Adapter);
			}
			else
			{
				// We don't have the link status yet since we are in the process of
				// determing it, so pend it.
				PWDM_NDIS_REQUEST	wdmNdisRequest = &Adapter->WdmNdisRequest;

				ASSERT (!MP_TEST_FLAG(Adapter, fMP_REQUEST_PENDING));

				SMSC_TRACE0(DBG_REQUEST,"Pending OID_GEN_LINK_SPEED...\r\n");

				wdmNdisRequest->Oid = Oid;
				wdmNdisRequest->RequestType = NdisRequestQueryInformation;
				wdmNdisRequest->InformationBuffer = InformationBuffer;
				wdmNdisRequest->InformationBufferLength = InformationBufferLength;
				wdmNdisRequest->BytesNeeded = BytesNeeded;
				wdmNdisRequest->BytesReadOrWritten = BytesWritten;

				Status = NDIS_STATUS_PENDING;
			}
#else
			ulInfo = NICGetLinkSpeed(Adapter);
#endif
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  %d \r\n", ulInfo);
			break;

		case OID_GEN_TRANSMIT_BUFFER_SPACE:
			//
			// Specify the amount of memory, in bytes, on the NIC that
			// is available for buffering transmit data. A protocol can
			// use this OID as a guide for sizing the amount of transmit
			// data per send.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_TRANSMIT_BUFFER_SPACE\r\n");
			ulInfo = Adapter->TxBurstMax * Adapter->MaxTCBs;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_RECEIVE_BUFFER_SPACE:
			//
			// Specify the amount of memory on the NIC that is available
			// for buffering receive data. A protocol driver can use this
			// OID as a guide for advertising its receive window after it
			// establishes sessions with remote nodes.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_RECEIVE_BUFFER_SPACE\r\n");
			ulInfo = Adapter->RxBurstMax * Adapter->MaxRCBs;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_VENDOR_ID:
			//
			// Specify a three-byte IEEE-registered vendor code, followed
			// by a single byte that the vendor assigns to identify a
			// particular NIC. The IEEE code uniquely identifies the vendor
			// and is the same as the three bytes appearing at the beginning
			// of the NIC hardware address. Vendors without an IEEE-registered
			// code should use the value 0xFFFFFF.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_VENDOR_ID\r\n");
			NdisMoveMemory(&ulInfo, Adapter->PermanentNodeAddress, 4);
			if (ulInfo == 0)
			{
				ulInfo = DEFAULT_NIC_VENDOR_ID;
			}
			ulInfo |= (DEFAULT_NIC_DEVICE_ID << 24);
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_VENDOR_DESCRIPTION:
			//
			// Specify a zero-terminated string describing the NIC vendor.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_VENDOR_DESCRIPTION\r\n");
			pInfo = VendorDesc;
			ulInfoLen = sizeof(VendorDesc);
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", VendorDesc);
			break;

		case OID_GEN_VENDOR_DRIVER_VERSION:
			//
			// Specify the vendor-assigned version number of the NIC driver.
			// The low-order half of the return value specifies the minor
			// version; the high-order half specifies the major version.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_VENDOR_DRIVER_VERSION\r\n");
			ulInfo = ((DRIVER_MAJOR_VERSION << 16) | DRIVER_MINOR_VERSION);
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_DRIVER_VERSION:
			//
			// Specify the NDIS version in use by the NIC driver. The high
			// byte is the major version number; the low byte is the minor
			// version number.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_DRIVER_VERSION\r\n");
			usInfo = (USHORT) (SMSC9500_NDIS_MAJOR_VERSION<<8) + SMSC9500_NDIS_MINOR_VERSION;
			pInfo = (PVOID) &usInfo;
			ulInfoLen = sizeof(USHORT);
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_MAXIMUM_SEND_PACKETS:
			//
			// If a miniport driver registers a MiniportSendPackets function,
			// MiniportQueryInformation will be called with the
			// OID_GEN_MAXIMUM_SEND_PACKETS request. The miniport driver must
			// respond with the maximum number of packets it is prepared to
			// handle on a single send request. The miniport driver should
			// pick a maximum that minimizes the number of packets that it
			// has to queue internally because it has no resources
			// (its device is full). A miniport driver for a bus-master DMA
			// NIC should attempt to pick a value that keeps its NIC filled
			// under anticipated loads.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_MAXIMUM_SEND_PACKETS\r\n");
			ulInfo = (Adapter->MaxTCBs * Adapter->TxBurstMax) / ETH_MAX_PACKET_SIZE;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_MEDIA_CONNECT_STATUS:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_MEDIA_CONNECT_STATUS\r\n");
#if 0 // Indicate media connection status immediatly. It fixes the issue of display getting delayed.
			if ((!Adapter->PerformingNWAY && !MP_TEST_FLAG(Adapter, fMP_RESET_IN_PROGRESS) && !MP_TEST_FLAG(Adapter, fMP_INIT_IN_PROGRESS))
				|| MP_TEST_FLAG(Adapter,fMP_DEVICE_NOT_ATTACHED))
			{
				//
				// Return the connection status of the NIC on the network as one
				// of the following system-defined values: NdisMediaStateConnected
				// or NdisMediaStateDisconnected.
				//
				ulInfo = NICGetMediaConnectStatus(Adapter);

			}
			else
			{
				// We don't have the link status yet since we are in the process of
				// determing it, so pend it.
				PWDM_NDIS_REQUEST	wdmNdisRequest = &Adapter->WdmNdisRequest;

				ASSERT (!MP_TEST_FLAG(Adapter, fMP_REQUEST_PENDING));

				SMSC_TRACE0(DBG_REQUEST,"Pending OID_GEN_MEDIA_CONNECT_STATUS...\r\n");

				wdmNdisRequest->Oid = Oid;
				wdmNdisRequest->RequestType = NdisRequestQueryInformation;
				wdmNdisRequest->InformationBuffer = InformationBuffer;
				wdmNdisRequest->InformationBufferLength = InformationBufferLength;
				wdmNdisRequest->BytesNeeded = BytesNeeded;
				wdmNdisRequest->BytesReadOrWritten = BytesWritten;

				Status = NDIS_STATUS_PENDING;
			}
#else
			ulInfo = NICGetMediaConnectStatus(Adapter);
#endif
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_CURRENT_PACKET_FILTER:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_CURRENT_PACKET_FILTER\r\n");
			//
			// Specifiy the types of net packets such as directed, broadcast
			// multicast, for which a protocol receives indications from a
			// NIC driver. After NIC is initialized, a protocol driver
			// can send a set OID_GEN_CURRENT_PACKET_FILTER to a non-zero value,
			// thereby enabling the miniport driver to indicate receive packets
			// to that protocol.
			//
			ulInfo = Adapter->PacketFilter;

			SMSC_TRACE0(DBG_REQUEST,"OID_GEN_CURRENT_PACKET_FILTER\r\n");
			#ifdef SMSC_DBG
			SMSC_TRACE0(DBG_REQUEST,"Reporting set NDIS Filters:\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_ALL_FUNCTIONAL)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_ALL_FUNCTIONAL\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_ALL_MULTICAST)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_ALL_MULTICAST\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_ALL_LOCAL)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_ALL_LOCAL\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_BROADCAST)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_BROADCAST\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_DIRECTED)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_DIRECTED\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_FUNCTIONAL)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_FUNCTIONAL\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_GROUP)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_GROUP\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_MAC_FRAME)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_MAC_FRAME\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_SMT)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_SMT\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_MULTICAST)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_MULTICAST\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_PROMISCUOUS)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_PROMISCUOUS\r\n");
			if( ulInfo & NDIS_PACKET_TYPE_SOURCE_ROUTING)
				SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_SOURCE_ROUTING\r\n");
			if( !(ulInfo & 0xFFFFFFFFUL))
				SMSC_TRACE0(DBG_REQUEST,"  NONE\r\n");
			if(
			   ulInfo & ~(
							NDIS_PACKET_TYPE_ALL_FUNCTIONAL |
							NDIS_PACKET_TYPE_ALL_MULTICAST |
							NDIS_PACKET_TYPE_ALL_LOCAL |
							NDIS_PACKET_TYPE_BROADCAST |
							NDIS_PACKET_TYPE_DIRECTED |
							NDIS_PACKET_TYPE_FUNCTIONAL |
							NDIS_PACKET_TYPE_GROUP |
							NDIS_PACKET_TYPE_MAC_FRAME |
							NDIS_PACKET_TYPE_SMT |
							NDIS_PACKET_TYPE_MULTICAST |
							NDIS_PACKET_TYPE_PROMISCUOUS |
							NDIS_PACKET_TYPE_SOURCE_ROUTING
					   )
			  )
			SMSC_TRACE1(DBG_REQUEST,"UNKNOWN FILTERS = %08X", ulInfo &\
							~(
								NDIS_PACKET_TYPE_ALL_FUNCTIONAL |
								NDIS_PACKET_TYPE_ALL_MULTICAST |
								NDIS_PACKET_TYPE_ALL_LOCAL |
								NDIS_PACKET_TYPE_BROADCAST |
								NDIS_PACKET_TYPE_DIRECTED |
								NDIS_PACKET_TYPE_FUNCTIONAL |
								NDIS_PACKET_TYPE_GROUP |
								NDIS_PACKET_TYPE_MAC_FRAME |
								NDIS_PACKET_TYPE_SMT |
								NDIS_PACKET_TYPE_MULTICAST |
								NDIS_PACKET_TYPE_PROMISCUOUS |
								NDIS_PACKET_TYPE_SOURCE_ROUTING
							)\
					);
			#endif
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);

			break;

		case OID_PNP_CAPABILITIES:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_PNP_CAPABILITIES\r\n");
			PMCaps.Flags = 0;	// They are reserved, but initialize to something...

			PMCaps.WakeUpCapabilities.MinMagicPacketWakeUp = \
				((Adapter->WolMagicPacketSettings != DISABLED) ? NdisDeviceStateD3 : NdisDeviceStateUnspecified);
			PMCaps.WakeUpCapabilities.MinPatternWakeUp = \
				((Adapter->WolPatternMatchSettings != DISABLED) ? NdisDeviceStateD3 : NdisDeviceStateUnspecified);
			PMCaps.WakeUpCapabilities.MinLinkChangeWakeUp = \
				((Adapter->WoLinkChgSettings != DISABLED) ? NdisDeviceStateD3 : NdisDeviceStateUnspecified);

			SMSC_TRACE1(DBG_REQUEST,"PM-Wakeup capabilities returned to NDIS:\r\n  Flags = %08lX\r\n", PMCaps.Flags);

			SMSC_TRACE0(DBG_REQUEST,"  MinMagicPacketWakeUp = ");
			switch(PMCaps.WakeUpCapabilities.MinMagicPacketWakeUp)
			{
				case NdisDeviceStateD0:
					SMSC_TRACE0(DBG_REQUEST,"D0\r\n");
					break;

				case NdisDeviceStateD1:
					SMSC_TRACE0(DBG_REQUEST,"D1\r\n");
					break;

				case NdisDeviceStateD2:
					SMSC_TRACE0(DBG_REQUEST,"D2\r\n");
					break;

				case NdisDeviceStateD3:
					SMSC_TRACE0(DBG_REQUEST,"D3\r\n");
					break;

				default:
					SMSC_TRACE0(DBG_REQUEST,"NdisDeviceStateUnspecified\r\n");
					break;
			}

			SMSC_TRACE0(DBG_REQUEST,"  MinPatternWakeUp = ");
			switch(PMCaps.WakeUpCapabilities.MinPatternWakeUp)
			{
				case NdisDeviceStateD0:
					SMSC_TRACE0(DBG_REQUEST,"D0\r\n");
					break;

				case NdisDeviceStateD1:
					SMSC_TRACE0(DBG_REQUEST,"D1\r\n");
					break;

				case NdisDeviceStateD2:
					SMSC_TRACE0(DBG_REQUEST,"D2\r\n");
					break;

				case NdisDeviceStateD3:
					SMSC_TRACE0(DBG_REQUEST,"D3\r\n");
					break;

				default:
					SMSC_TRACE0(DBG_REQUEST,"NdisDeviceStateUnspecified\r\n");
					break;
			}

			SMSC_TRACE0(DBG_REQUEST,"  MinLinkChangeWakeUp = ");
			switch(PMCaps.WakeUpCapabilities.MinLinkChangeWakeUp)
			{
				case NdisDeviceStateD0:
					SMSC_TRACE0(DBG_REQUEST,"D0\r\n");
					break;

				case NdisDeviceStateD1:
					SMSC_TRACE0(DBG_REQUEST,"D1\r\n");
					break;

				case NdisDeviceStateD2:
					SMSC_TRACE0(DBG_REQUEST,"D2\r\n");
					break;

				case NdisDeviceStateD3:
					SMSC_TRACE0(DBG_REQUEST,"D3\r\n");
					break;

				default:
					SMSC_TRACE0(DBG_REQUEST,"NdisDeviceStateUnspecified\r\n");
					break;
			}

			if (Adapter->PowerManagement != DISABLED)
			{
				ulInfoLen = sizeof (NDIS_PNP_CAPABILITIES);
				pInfo = (PVOID) &PMCaps;
				Status = NDIS_STATUS_SUCCESS;
			}
			else
			{
				Status = NDIS_STATUS_NOT_SUPPORTED;
			}
			break;

		case OID_PNP_QUERY_POWER:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_PNP_QUERY_POWER\r\n");
			if (InformationBufferLength == 0)
			{
				Status = NDIS_STATUS_INVALID_LENGTH;
				break;
			}

			SMSC_TRACE0(DBG_REQUEST,"Power State query of 'readiness' to change to state D\r\n");
			switch (*((PNDIS_DEVICE_POWER_STATE)InformationBuffer))
			{
				case NdisDeviceStateD0:
					SMSC_TRACE0(DBG_REQUEST,"0\r\n");
					break;
				case NdisDeviceStateD1:
					SMSC_TRACE0(DBG_REQUEST,"1\r\n");
					break;
				case NdisDeviceStateD2:
					SMSC_TRACE0(DBG_REQUEST,"2\r\n");
					break;
				case NdisDeviceStateD3:
					SMSC_TRACE0(DBG_REQUEST,"3\r\n");
					break;
				default:
					SMSC_TRACE0(DBG_REQUEST,"? - Unknown - Returning OK anyway\r\n");
					break;
			}

			if (Adapter->PowerManagement != DISABLED)
			{
				Status = NDIS_STATUS_SUCCESS;
			}
			else
			{
				Status = NDIS_STATUS_NOT_SUPPORTED;
			}

			break;

		case OID_PNP_ENABLE_WAKE_UP:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_PNP_ENABLE_WAKE_UP\r\n");
			ulInfo = 0;
			
			if(
				(Adapter->PowerManagement != DISABLED) &&
			   (
				(Adapter->WolPatternMatchSettings != DISABLED) ||
				(Adapter->WolMagicPacketSettings != DISABLED) ||
				(Adapter->WoLinkChgSettings != DISABLED)
			   )
			  )
			{
				if (Adapter->CurrentWakeEvents & NIC_FEATURE_MAGIC_PACKET)
				{
					ulInfo |= NDIS_PNP_WAKE_UP_MAGIC_PACKET;
					SMSC_TRACE0(DBG_REQUEST,"  Magic Packet\r\n");
				}

				if (Adapter->CurrentWakeEvents & NIC_FEATURE_WAKE_ON_LAN)
				{
					ulInfo |= NDIS_PNP_WAKE_UP_PATTERN_MATCH;
					SMSC_TRACE0(DBG_REQUEST,"  Wake On LAN\r\n");
				}

				if (Adapter->CurrentWakeEvents & NIC_FEATURE_LINK_CHANGE)
				{
					ulInfo |= NDIS_PNP_WAKE_UP_LINK_CHANGE;
					SMSC_TRACE0(DBG_REQUEST,"  Link Change\r\n");
				}

				Status = NDIS_STATUS_SUCCESS;
			}
			else
			{
				SMSC_TRACE0(DBG_REQUEST,"OID_PNP_QUERY_POWER is turned off or not supported by this device\r\n");
				Status = NDIS_STATUS_NOT_SUPPORTED;
			}
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);

			break;

			
			//
			// Following 4 OIDs are for querying Ethernet Operational
			// Characteristics.
			//
		case OID_802_3_PERMANENT_ADDRESS:
			//
			// Return the MAC address of the NIC burnt in the hardware.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_PERMANENT_ADDRESS\r\n");
			pInfo = Adapter->PermanentNodeAddress;
			ulInfoLen = ETH_LENGTH_OF_ADDRESS;
			#ifdef SMSC_DBG
			if (g_DebugMode & DBG_REQUEST) 
			{
				int j;

				SMSC_TRACE0(DBG_REQUEST,"PermanentNodeAddress :");
				for(j = 0; j < 6; j++)
					RETAILMSG(1,(TEXT(" %02x"), Adapter->PermanentNodeAddress[j]));
				RETAILMSG(1,(TEXT("\r\n")));
			}
			#endif
			

			break;

		case OID_802_3_CURRENT_ADDRESS:
			//
			// Return the MAC address the NIC is currently programmed to
			// use. Note that this address could be different from the
			// permananent address as the user can override using
			// registry. Read NdisReadNetworkAddress doc for more info.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_CURRENT_ADDRESS\r\n");
			pInfo = Adapter->CurrentNodeAddress;
			ulInfoLen = ETH_LENGTH_OF_ADDRESS;
			#ifdef SMSC_DBG
			if (g_DebugMode & DBG_REQUEST) 
			{
				int j;

				SMSC_TRACE0(DBG_REQUEST,"CurrentNodeAddress :");
				for(j = 0; j < 6; j++)
					RETAILMSG(1,(TEXT(" %02x"), Adapter->CurrentNodeAddress[j]));
				RETAILMSG(1,(TEXT("\r\n")));
			}
			#endif
			
			break;

		case OID_802_3_MULTICAST_LIST:



			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_MULTICAST_LIST\r\n");
			pInfo = Adapter->MCList;
			ulInfoLen = Adapter->ulMCListSize * ETH_LENGTH_OF_ADDRESS;
			break;

		case OID_802_3_MAXIMUM_LIST_SIZE:
			//
			// The maximum number of multicast addresses the NIC driver
			// can manage. This list is global for all protocols bound
			// to (or above) the NIC. Consequently, a protocol can receive
			// NDIS_STATUS_MULTICAST_FULL from the NIC driver when
			// attempting to set the multicast address list, even if
			// the number of elements in the given list is less than
			// the number originally returned for this query.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_MAXIMUM_LIST_SIZE\r\n");
			ulInfo = NIC_MAX_MCAST_LIST;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);

			break;

		case OID_802_3_MAC_OPTIONS:
			//
			// Return zero indicating that it supports no options.
			//
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_MAC_OPTIONS\r\n");
			ulInfo = 0;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

			//
			// Following list  consists of both general and Ethernet
			// specific statistical OIDs.
			//

		case OID_GEN_XMIT_OK:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_XMIT_OK\r\n");
			ulInfo64 = Adapter->nPacketsTransmittedSuccessfully;
			pInfo = &ulInfo64;
			if (InformationBufferLength >= sizeof(ULONG64) ||
				InformationBufferLength == 0)
			{
				ulInfoLen = sizeof(ULONG64);
			}
			else
			{
				ulInfoLen = sizeof(ULONG);
			}
			// We should always report that 8 bytes are required to keep ndistest happy
			*BytesNeeded =	sizeof(ULONG64);

			break;

		case OID_GEN_RCV_OK:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_RCV_OK\r\n");
			ulInfo64 = Adapter->nPacketsIndicatedOnRx;
			pInfo = &ulInfo64;
			if (InformationBufferLength >= sizeof(ULONG64) ||
				InformationBufferLength == 0)
			{
				ulInfoLen = sizeof(ULONG64);
			}
			else
			{
				ulInfoLen = sizeof(ULONG);
			}
			// We should always report that 8 bytes are required to keep ndistest happy
			*BytesNeeded =	sizeof(ULONG64);
			break;

		case OID_GEN_XMIT_ERROR:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_XMIT_ERROR\r\n");
			ulInfo = Adapter->TxAbortExcessCollisions +
				Adapter->TxDmaUnderrun +
				Adapter->TxLostCRS +
				Adapter->TxLateCollisions+
				Adapter->TransmitFailuresOther+
				Adapter->nBulkOutCompletedWithError;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_RCV_ERROR:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_RCV_ERROR\r\n");
			ulInfo = Adapter->RcvCrcErrors +
				Adapter->RcvAlignmentErrors +
				Adapter->RcvResourceErrors +
				Adapter->RcvDmaOverrunErrors +
				Adapter->RcvRuntErrors;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_RCV_NO_BUFFER:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_RCV_NO_BUFFER\r\n");
			ulInfo = Adapter->RcvResourceErrors;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_RCV_CRC_ERROR:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_RCV_CRC_ERROR\r\n");
			ulInfo = Adapter->RcvCrcErrors;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_GEN_TRANSMIT_QUEUE_LENGTH:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_TRANSMIT_QUEUE_LENGTH\r\n");
			ulInfo = Adapter->nTcbBusySend;	// Accurate for single pkt mode since in multipkt each URB can have more than 1 pkt.
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_802_3_RCV_ERROR_ALIGNMENT:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_RCV_ERROR_ALIGNMENT\r\n");
			ulInfo = Adapter->RcvAlignmentErrors;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_802_3_XMIT_ONE_COLLISION:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_XMIT_ONE_COLLISION\r\n");
			ulInfo = Adapter->OneRetry;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_802_3_XMIT_MORE_COLLISIONS:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_XMIT_MORE_COLLISIONS\r\n");
			ulInfo = Adapter->MoreThanOneRetry;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_802_3_XMIT_DEFERRED:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_XMIT_DEFERRED\r\n");
			ulInfo = Adapter->TxOKButDeferred;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_802_3_XMIT_MAX_COLLISIONS:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_XMIT_MAX_COLLISIONS\r\n");
			ulInfo = Adapter->TxAbortExcessCollisions;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_802_3_RCV_OVERRUN:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_RCV_OVERRUN\r\n");
			ulInfo = Adapter->RcvDmaOverrunErrors;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_802_3_XMIT_UNDERRUN:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_XMIT_UNDERRUN\r\n");
			ulInfo = Adapter->TxDmaUnderrun;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_802_3_XMIT_HEARTBEAT_FAILURE:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_XMIT_HEARTBEAT_FAILURE\r\n");
			ulInfo = Adapter->TxLostCRS;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_802_3_XMIT_TIMES_CRS_LOST:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_GEN_HARDWARE_STATUS\r\n");
			ulInfo = Adapter->TxLostCRS;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

		case OID_802_3_XMIT_LATE_COLLISIONS:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_802_3_XMIT_LATE_COLLISIONS\r\n");
			ulInfo = Adapter->TxLateCollisions;
			SMSC_TRACE1(DBG_REQUEST,"ulInfo  0x%x \r\n", ulInfo);
			break;

	case OID_NDIS_SMSC_LAN_READ_REGS:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_NDIS_SMSC_LAN_READ_REGS\r\n");
			{
				USHORT index=*(USHORT *)InformationBuffer;
				if (NICReadRegister( Adapter, index, &ulInfo) ==NIC_STATUS_SUCCESS) {
					ulInfoLen=sizeof(DWORD);
					pInfo=(PVOID)&ulInfo;
	   			}  else {
					Status = NDIS_STATUS_FAILURE;
	   			}
			}
			break;
	case OID_NDIS_SMSC_PHY_READ_REGS:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_NDIS_SMSC_PHY_READ_REGS\r\n");
			{
				DWORD index=*(DWORD *)InformationBuffer;
				if (Adapter->MdioDisable & MDIO_DISABLE_IOCTL) {
					Status = NDIS_STATUS_FAILURE;
					break;
				}
				if (NICReadPhy( Adapter, index, &ulInfo) == NIC_STATUS_SUCCESS) {
					ulInfoLen=sizeof(DWORD);
					pInfo=(PVOID)&ulInfo;
	   			}  else {
					Status = NDIS_STATUS_FAILURE;
	   			}
			}
			break;

	case OID_NDIS_SMSC_PHY_READ_REG_ADDRESS:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_NDIS_SMSC_PHY_READ_REG_ADDRESS\r\n");
			{
				DWORD phyAddress=(*(DWORD *)InformationBuffer) & 0x1F;
				DWORD index=*(DWORD *)((DWORD *)InformationBuffer+1);
				DWORD ReadDword=*(DWORD *)((DWORD *)InformationBuffer+2);
				if (Adapter->MdioDisable & MDIO_DISABLE_IOCTL) {
					Status = NDIS_STATUS_FAILURE;
					break;
				}
				if (NICReadPhyAddress(Adapter, phyAddress, index, &ulInfo, ReadDword) == NIC_STATUS_SUCCESS) {
					ulInfoLen=sizeof(DWORD);
					pInfo=(PVOID)&ulInfo;
	   			}  else {
					Status = NDIS_STATUS_FAILURE;
	   			}
			}
			break;	
	case OID_NDIS_SMSC_GET_DRIVER_INFO:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_NDIS_SMSC_GET_DRIVER_INFO\r\n");
			{
				DRIVER_VERSION_STRUCT DriVer;
		
				DriVer.Major=DRIVER_MAJOR_VERSION;
				DriVer.Minor=DRIVER_MINOR_VERSION;
				DriVer.Build=DRIVER_BUILD_VERSION;
				DriVer.Qfe=DRIVER_QFE_VERSION;
				pInfo= (PVOID)&DriVer;
				ulInfoLen=sizeof(DriVer);			
			}
			break;

	 case OID_NDIS_SMSC_READ_E2P:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_NDIS_SMSC_READ_E2P\r\n");
	 	     	{
				WORD Address=*(WORD *)InformationBuffer;			
				BYTE ValueByte;
				if (NICReadEepromByte(Adapter,Address,&ValueByte) ==NIC_STATUS_SUCCESS) {
					ulInfoLen=sizeof(BYTE);
					pInfo=(PVOID)&ValueByte;
	   			}  else {
					Status = NDIS_STATUS_FAILURE;
	   			}
	 		}
	 		break;
	 case OID_NDIS_SMSC_READ_E2P_BLOCK:
			SMSC_TRACE0(DBG_REQUEST,"Query OID_NDIS_SMSC_READ_E2P_BLOCK\r\n");
			{
			 						 
				OID_E2P_STRUCT eepromStruct;
				eepromStruct.E2pSize=*(WORD *)InformationBuffer;
										
				if (NICReadEepromBlock(Adapter,0,eepromStruct.E2pSize,	(PBYTE)eepromStruct.eeprom)!=NIC_STATUS_SUCCESS) {		
					Status = Status = NDIS_STATUS_FAILURE;
					break;
				}
				pInfo = (PVOID)&eepromStruct;
				ulInfoLen = sizeof(eepromStruct);		
				}
			break;

	case OID_NDIS_SMSC_GET_LINK_STATUS:
			SMSC_TRACE0(DBG_REQUEST,"Set OID_NDIS_SMSC_GET_LINK_STATUS\r\n");
			{
				if (MP_TEST_FLAG(Adapter, fMP_DISCONNECTED))
				{
					ulInfo = 0;
				} else {
					ulInfo = 1;
				}
				ulInfoLen = sizeof(DWORD);
				pInfo = (PVOID)&ulInfo;
			}
			break;				
		default:
			Status = NDIS_STATUS_NOT_SUPPORTED;
			break;
	}

	if ((Status == NDIS_STATUS_SUCCESS) || (Status == NDIS_STATUS_PENDING))
	{
		if(ulInfoLen > InformationBufferLength)
		{
			// too short
			*BytesNeeded = ulInfoLen;
			Status = NDIS_STATUS_INVALID_LENGTH;
		}
		else
		{
			if (Status == NDIS_STATUS_SUCCESS)
			{
				// Copy result into InformationBuffer
				*BytesWritten = ulInfoLen;
				if(ulInfoLen)
				{
					if (InformationBuffer != pInfo) {
						NdisMoveMemory(InformationBuffer, pInfo, ulInfoLen);
					}
				}
			}

			if (Status == NDIS_STATUS_PENDING)
			{
				// Set the pending flag
				MP_SET_FLAG(Adapter, fMP_REQUEST_PENDING);
			}
		}
	}

Exit_MPQueryInformation:
	SMSC_TRACE2(DBG_REQUEST,"<-- MPQueryInformation 0x%x, Status = 0x%x\r\n",  Oid, Status);

	NdisReleaseSpinLock(&Adapter->RequestLock);

	return(Status);
}

/*++

Routine Description:

	This is the handler for an OID set operation.
	

Arguments:

	MiniportAdapterContext			Pointer to the adapter structure
	Oid 									Oid for this query
	InformationBuffer					Buffer for information
	InformationBufferLength 		Size of this buffer
	BytesRead							Specifies how much info is read
	BytesNeeded 						In case the buffer is smaller than what
											we need, tell them how much is needed

Return Value:

	Return code from the NdisRequest below.

--*/
NDIS_STATUS
MPSetInformation(
	IN NDIS_HANDLE									MiniportAdapterContext,
	IN NDIS_OID 									Oid,
	__in_bcount(InformationBufferLength) IN PVOID	InformationBuffer,
	IN ULONG										InformationBufferLength,
	OUT PULONG										BytesRead,
	OUT PULONG										BytesNeeded)

{
	NDIS_STATUS 			Status = NDIS_STATUS_SUCCESS;
	PMP_ADAPTER 			Adapter = (PMP_ADAPTER) MiniportAdapterContext;
	

	if(Adapter == NULL){
		SMSC_TRACE0(DBG_ERROR,"MPSetInformation Adapter = NULL\r\n");
		return NDIS_STATUS_FAILURE;
	}
	NdisAcquireSpinLock(&Adapter->RequestLock);

	SMSC_TRACE1(DBG_REQUEST,"--> MPSetInformation 0x%08lx\r\n", Oid);

	*BytesRead = 0;
	*BytesNeeded = 0;

	if (MP_TEST_FLAG(Adapter, fMP_SURPRISE_REMOVED))
	{
		Status = NDIS_STATUS_NOT_ACCEPTED;
		goto Exit_MPSetInformation;
	}

	switch(Oid)
	{
		case OID_802_3_MULTICAST_LIST:
			//
			// Set the multicast address list on the NIC for packet reception.
			// The NIC driver can set a limit on the number of multicast
			// addresses bound protocol drivers can enable simultaneously.
			// NDIS returns NDIS_STATUS_MULTICAST_FULL if a protocol driver
			// exceeds this limit or if it specifies an invalid multicast
			// address.
			//
			SMSC_TRACE0(DBG_REQUEST,"Set OID_802_3_MULTICAST_LIST\r\n");
			Status = NICSetMulticastList(
							Adapter,
							InformationBuffer,
							InformationBufferLength,
							BytesRead,
							BytesNeeded);
			break;

		case OID_GEN_CURRENT_PACKET_FILTER:
			//
			// Program the hardware to indicate the packets
			// of certain filter types.
			//
			SMSC_TRACE0(DBG_REQUEST,"Set OID_GEN_CURRENT_PACKET_FILTER\r\n");

			if(InformationBufferLength != sizeof(ULONG))
			{
				SMSC_TRACE1(DBG_REQUEST,"Invalid buffer length for filter: %d\r\n",
									InformationBufferLength);
				*BytesNeeded = sizeof(ULONG);
				Status = NDIS_STATUS_INVALID_LENGTH;
				break;
			}

			*BytesRead = InformationBufferLength;

			Status = NICSetPacketFilter(
							Adapter,
							*((PULONG)InformationBuffer));

			break;

		case OID_GEN_CURRENT_LOOKAHEAD:
			//
			// A protocol driver can set a suggested value for the number
			// of bytes to be used in its binding; however, the underlying
			// NIC driver is never required to limit its indications to
			// the value set.
			//
			SMSC_TRACE0(DBG_REQUEST,"Set OID_GEN_CURRENT_LOOKAHEAD\r\n");
			if(InformationBufferLength != sizeof(ULONG)){
				*BytesNeeded = sizeof(ULONG);
				Status = NDIS_STATUS_INVALID_LENGTH;
				break;
			}

			Adapter->ulLookAhead = *(PULONG)InformationBuffer;

			*BytesRead = sizeof(ULONG);
			Status = NDIS_STATUS_SUCCESS;

			break;

		case OID_PNP_SET_POWER:
			//
			// This OID notifies a miniport driver that its NIC will be
			// transitioning to the device power state specified in the
			// InformationBuffer. The miniport driver must always return
			// NDIS_STATUS_SUCCESS to an OID_PNP_SET_POWER request. An
			// OID_PNP_SET_POWER request may or may not be preceded by an
			// OID_PNP_QUERY_POWER request.
			//
			SMSC_TRACE0(DBG_REQUEST,"Set OID_PNP_SET_POWER\r\n");
			if (InformationBufferLength != sizeof(NDIS_DEVICE_POWER_STATE ))
			{
				Status = NDIS_STATUS_INVALID_LENGTH;
				break;
			}
			*BytesRead = sizeof(NDIS_DEVICE_POWER_STATE);

			if (MP_TEST_FLAG(Adapter,fMP_DEVICE_NOT_ATTACHED)) {
				Status = NDIS_STATUS_FAILURE;
				break;
			}
			
			if (Adapter->PowerManagement != DISABLED)
			{
				SMSC_TRACE0(DBG_REQUEST,"Power state switching scheduled.\r\n");
				Status = NDIS_STATUS_PENDING;
			}
			else
			{
				SMSC_TRACE0(DBG_REQUEST,"Power state switching disabled or not supported by this device\r\n");
				Status = NDIS_STATUS_NOT_SUPPORTED;
			}

			break;

		case OID_PNP_ADD_WAKE_UP_PATTERN:
			//
			// This OID is sent by a protocol driver to a miniport driver to
			// specify a wake-up pattern. The wake-up pattern, along with its mask,
			// is described by an NDIS_PM_PACKET_PATTERN structure.
			//
			SMSC_TRACE0(DBG_REQUEST,"Set OID_PNP_ADD_WAKE_UP_PATTERN\r\n");
			if (Adapter->WolPatternMatchSettings != DISABLED)
			{
				PNDIS_PM_PACKET_PATTERN pNdisWolSample;
				LM_WOL_SAMPLE_STRUCT LMWolSample, *pLMWolStore;
				WORD i, Count, First, Last;
				BYTE HeadPattern;

				pNdisWolSample = (PNDIS_PM_PACKET_PATTERN) InformationBuffer;

				if (InformationBufferLength < sizeof(NDIS_PM_PACKET_PATTERN))
				{
					Status = NDIS_STATUS_BUFFER_TOO_SHORT;

					*BytesNeeded = sizeof(NDIS_PM_PACKET_PATTERN);
					break;
				}
				if (InformationBufferLength < pNdisWolSample->PatternOffset + pNdisWolSample->PatternSize)
				{
					Status = NDIS_STATUS_BUFFER_TOO_SHORT;

					*BytesNeeded = pNdisWolSample->PatternOffset + pNdisWolSample->PatternSize;
					break;
				}
				*BytesRead = pNdisWolSample->PatternOffset + pNdisWolSample->PatternSize;

				LMWolSample.pWOLSample = ((PUCHAR) InformationBuffer) + pNdisWolSample->PatternOffset;
				LMWolSample.WOLSampleSize = pNdisWolSample->PatternSize;
				LMWolSample.pWOLMask = ((PUCHAR) InformationBuffer) + sizeof(NDIS_PM_PACKET_PATTERN);
				LMWolSample.WOLMaskSize = pNdisWolSample->MaskSize;

				// If we already have this pattern do not load it again

				if (NICFindWolSample(Adapter, &LMWolSample) != NULL)
				{
					SMSC_TRACE0(DBG_REQUEST,"WOL Pattern already stored! Not accepting it.\r\n");

					Status = NDIS_STATUS_INVALID_DATA;
					break;
				}

				// Do we have any WOL patterns available ?

				if(Adapter->WOLPatternsInUse == Adapter->NumOfWolPatterns)
				{
					SMSC_TRACE0(DBG_REQUEST,"All the WOL Patterns are in use !\r\n");

					Status = NDIS_STATUS_RESOURCES;
					break;
				}

				// Lets figure out how big the sample is and where it starts from the beginning
				// of the frame.

				First = Last = Count = 0;

				for(i=0;i<LMWolSample.WOLSampleSize;i++)
				{
					if( LMWolSample.pWOLMask[i/8] & (1 << (i%8)) )
					{
						Count++;
						Last = i;
					}

					if(!Count)
						First++;
				}

				if (!Count)
				{
					SMSC_TRACE0(DBG_REQUEST,"Invalid Pattern. The mask disables all bytes !\r\n");

					Status = NDIS_STATUS_INVALID_DATA;
					break;
				}

				// Is the first enabled byte within MAX_OFFSET_OF_WOL_PATTERN (255) bytes of the start of the packet ?
				if (First > MAX_OFFSET_OF_WOL_PATTERN)
				{
					SMSC_TRACE1(DBG_REQUEST,"Pattern starts after %d bytes, cannot add it!\r\n", MAX_OFFSET_OF_WOL_PATTERN);

					Status = NDIS_STATUS_NOT_ACCEPTED ;
					break;
				}

				if ( (Last - First + 1) > SIZE_OF_WOL_PATTERN)
				{
					// Our hardware cannot do complete matching for this pattern, but
					// let's do the best that we can (wake up when we match the first part)
					SMSC_TRACE2(DBG_REQUEST,"Pattern too big (%d), will add it with partial match up to the first %d bytes starting at offset!\r\n", (Last - First + 1), SIZE_OF_WOL_PATTERN);
				}

				// Get the Head Pattern, set pLMWolSStore to point to its corresponding
				//	structure and fill it up.

				HeadPattern = NICGetWOLPattern(Adapter);
				if (HeadPattern == Adapter->NumOfWolPatterns)
				{
					SMSC_TRACE0(DBG_REQUEST,"ERROR: LM_GetWOLPattern should have passed but failed\r\n");
					Status = NDIS_STATUS_NOT_ACCEPTED ;
					break;
				}

				pLMWolStore = (PLM_WOL_SAMPLE_STRUCT) (Adapter->NicWolMemBlock + (HeadPattern * sizeof(LM_WOL_SAMPLE_STRUCT)));
				
				NdisMoveMemory((PVOID) pLMWolStore->pWOLSample, (PVOID) LMWolSample.pWOLSample, LMWolSample.WOLSampleSize);
				NdisMoveMemory((PVOID) pLMWolStore->pWOLMask, (PVOID) LMWolSample.pWOLMask, LMWolSample.WOLMaskSize);

				pLMWolStore->HeadPattern	= HeadPattern;
				pLMWolStore->WOLSampleSize	= LMWolSample.WOLSampleSize;
				pLMWolStore->WOLMaskSize	= LMWolSample.WOLMaskSize;
				pLMWolStore->FirstEnabledByte	= First;
				pLMWolStore->LastEnabledByte	= Last;

				// Setup WOL Sample in chip.
				Status = NICAddWOLSample(Adapter, pLMWolStore);
			}
			else
			{
				SMSC_TRACE0(DBG_REQUEST,"Wake On LAN disabled or not supported by this device\r\n");
				Status = NDIS_STATUS_NOT_SUPPORTED;
			}

			break;

		case OID_PNP_REMOVE_WAKE_UP_PATTERN:
			//
			// This OID requests the miniport driver to delete a wake-up pattern
			// that it previously received in an OID_PNP_ADD_WAKE_UP_PATTERN request.
			// The wake-up pattern, along with its mask, is described by an
			// NDIS_PM_PACKET_PATTERN structure.
			//
			SMSC_TRACE0(DBG_REQUEST,"Set OID_PNP_REMOVE_WAKE_UP_PATTERN \r\n");
			if (Adapter->WolPatternMatchSettings != DISABLED)
			{
				PNDIS_PM_PACKET_PATTERN pNdisWolSample;
				LM_WOL_SAMPLE_STRUCT LMWolSample, *pLMWolStore;

		
				pNdisWolSample = (PNDIS_PM_PACKET_PATTERN) InformationBuffer;

				if (InformationBufferLength < sizeof(NDIS_PM_PACKET_PATTERN))
				{
					Status = NDIS_STATUS_BUFFER_TOO_SHORT;

					*BytesNeeded = sizeof(NDIS_PM_PACKET_PATTERN);
					break;
				}
				if (InformationBufferLength < pNdisWolSample->PatternOffset + pNdisWolSample->PatternSize)
				{
					Status = NDIS_STATUS_BUFFER_TOO_SHORT;

					*BytesNeeded = pNdisWolSample->PatternOffset + pNdisWolSample->PatternSize;
					break;
				}
				*BytesRead = pNdisWolSample->PatternOffset + pNdisWolSample->PatternSize;

				LMWolSample.pWOLSample = ((PUCHAR) InformationBuffer) + pNdisWolSample->PatternOffset;
				LMWolSample.WOLSampleSize = pNdisWolSample->PatternSize;
				LMWolSample.pWOLMask = ((PUCHAR) InformationBuffer) + sizeof(NDIS_PM_PACKET_PATTERN);
				LMWolSample.WOLMaskSize = pNdisWolSample->MaskSize;

				// Check if there are any patterns enabled

				if (Adapter->WOLPatternsInUse == 0)
				{
					SMSC_TRACE0(DBG_REQUEST,"There are no WOL packets enabled !\r\n");
					Status = NDIS_STATUS_FAILURE;
					break;
				}

				// Search for matching sample now

				pLMWolStore = NICFindWolSample(Adapter, &LMWolSample);

				if (pLMWolStore != NULL)
				{
					// Got it => Remove it
					NICRemoveWOLSample(Adapter, pLMWolStore);
					Status = NDIS_STATUS_SUCCESS;
				}
				else
				{
					// We don't have it => fail the request.
					Status = NDIS_STATUS_FAILURE;
				}
			}
			else
			{
				SMSC_TRACE0(DBG_REQUEST,"Wake On LAN disabled or not supported by this device\r\n");
				Status = NDIS_STATUS_NOT_SUPPORTED;
			}

			break;


		case OID_PNP_ENABLE_WAKE_UP:
			//
			// This OID specifies which wake-up capabilities a miniport
			// driver should enable in its NIC. Before the miniport
			// transitions to a low-power state (that is, before NDIS
			// sends the miniport driver an OID_PNP_SET_POWER request),
			// NDIS sends the miniport an OID_PNP_ENABLE_WAKE_UP request to
			// enable the appropriate wake-up capabilities.
			//
			SMSC_TRACE0(DBG_REQUEST,"Set OID_PNP_ENABLE_WAKE_UP\r\n");
			if(InformationBufferLength != sizeof(ULONG))
			{
				*BytesNeeded = sizeof(ULONG);
				Status = NDIS_STATUS_INVALID_LENGTH;
				break;
			}
			*BytesRead = sizeof(ULONG);

			if(
				(Adapter->PowerManagement != DISABLED) &&
				(
					(Adapter->WolPatternMatchSettings != DISABLED) ||
					(Adapter->WolMagicPacketSettings != DISABLED) ||
					(Adapter->WoLinkChgSettings != DISABLED)
				)
			  )
			{
			  if (Adapter->CurrentPowerState == 0) // If already in power down skip
			  {
				ULONG Flags;

				Flags = *((PULONG) InformationBuffer);

				SMSC_TRACE0(DBG_REQUEST,"NDIS Requests to enable the following Wake up events:\r\n");
				#ifdef SMSC_DBG
				if (Flags & NDIS_PNP_WAKE_UP_MAGIC_PACKET)
						SMSC_TRACE0(DBG_REQUEST,"  Magic Packet\r\n");
				if (Flags & NDIS_PNP_WAKE_UP_PATTERN_MATCH)
						SMSC_TRACE0(DBG_REQUEST,"  Wake On LAN\r\n");
				if (Flags & NDIS_PNP_WAKE_UP_LINK_CHANGE)
						SMSC_TRACE0(DBG_REQUEST,"  Link Change\r\n");
				if (!(Flags & (NDIS_PNP_WAKE_UP_MAGIC_PACKET | NDIS_PNP_WAKE_UP_PATTERN_MATCH | NDIS_PNP_WAKE_UP_LINK_CHANGE)))
						SMSC_TRACE0(DBG_REQUEST,"  None\r\n");
				#endif

				Adapter->CurrentWakeEvents = 0UL;

				if (Flags & NDIS_PNP_WAKE_UP_MAGIC_PACKET)
				{
					if (Adapter->WolMagicPacketSettings != DISABLED)
					{
						SMSC_TRACE0(DBG_REQUEST,"Enabling Magic Packet\r\n");
						Adapter->CurrentWakeEvents |= NIC_FEATURE_MAGIC_PACKET;
					}
					else
					{
						SMSC_TRACE0(DBG_REQUEST,"Magic Packet disabled or not supported by this device\r\n");
					}
				}

				if (Flags & NDIS_PNP_WAKE_UP_PATTERN_MATCH)
				{
					if (Adapter->WolPatternMatchSettings != DISABLED)
					{
						SMSC_TRACE0(DBG_REQUEST,"Enabling Wake On LAN\r\n");
						Adapter->CurrentWakeEvents |= NIC_FEATURE_WAKE_ON_LAN;
					}
					else
					{
						SMSC_TRACE0(DBG_REQUEST,"Wake On Lan not supported by this device\r\n");
					}
				}

				if (Flags & NDIS_PNP_WAKE_UP_LINK_CHANGE)
				{
					if (Adapter->WoLinkChgSettings != DISABLED)
					{
						SMSC_TRACE0(DBG_REQUEST,"Enabling Link Change\r\n");
						Adapter->CurrentWakeEvents |= NIC_FEATURE_LINK_CHANGE;
					}
					else
					{
						SMSC_TRACE0(DBG_REQUEST,"Link change not supported by this device\r\n");
					}
				}

				// Note that setting up the actual hardware for wakes is NOT done here
				// but performed at the call to do to the sleep state instead since (i.e.)
				// MP & pattern match CANNOT be enabled at this point without disturbing
				// normal reception
			  }
			  else
			  {
				SMSC_TRACE0(DBG_REQUEST,"We are already in Power Down. Just return SUCCESS without touching the chip\r\n");
			  }

			  Status = NDIS_STATUS_SUCCESS;
			}
			else
			{
			  Status = NDIS_STATUS_NOT_SUPPORTED;
			}

			break;

	case OID_NDIS_SMSC_LAN_WRITE_REGS:
			SMSC_TRACE0(DBG_REQUEST,"Set OID_NDIS_SMSC_LAN_WRITE_REGS\r\n");	
			{
				DWORD index=*(DWORD *)InformationBuffer;		
				DWORD dwValue=*((DWORD *)InformationBuffer+1);
				if (NICWriteRegister( Adapter, (USHORT)index, &dwValue) !=NIC_STATUS_SUCCESS)
				 {
					Status = NDIS_STATUS_FAILURE;
				 }
			}
			break;

	case OID_NDIS_SMSC_PHY_WRITE_REGS:
			SMSC_TRACE0(DBG_REQUEST,"Set OID_NDIS_SMSC_PHY_WRITE_REGS\r\n");
			{
				DWORD index=*(DWORD *)InformationBuffer;
				DWORD dwValue=*(DWORD *)((DWORD *)InformationBuffer+1);
				if (Adapter->MdioDisable & MDIO_DISABLE_IOCTL) {
					Status = NDIS_STATUS_FAILURE;
					break;
				}
				if (NICWritePhy( Adapter, index, dwValue) !=NIC_STATUS_SUCCESS) {
					Status = NDIS_STATUS_FAILURE;
	   			}
			}
			break;

	case OID_NDIS_SMSC_PHY_WRITE_REG_ADDRESS:
			SMSC_TRACE0(DBG_REQUEST,"Set OID_NDIS_SMSC_PHY_WRITE_REG_ADDRESS\r\n");
			{
				DWORD phyAddress=(*(DWORD *)InformationBuffer) & 0x1F;
				DWORD index=*(DWORD *)((DWORD *)InformationBuffer+1);
				DWORD dwValue=*(DWORD *)((DWORD *)InformationBuffer+2);
				DWORD WriteDword=*(DWORD *)((DWORD *)InformationBuffer+3);
				if (Adapter->MdioDisable & MDIO_DISABLE_IOCTL) {
					Status = NDIS_STATUS_FAILURE;
					break;
				}
				if (NICWritePhyAddress(Adapter, phyAddress, index, dwValue, WriteDword) != NIC_STATUS_SUCCESS) {
					Status = NDIS_STATUS_FAILURE;
	   			}
			}
			break;

	case OID_NDIS_SMSC_SEND_E2P_COMMAND:	
			SMSC_TRACE0(DBG_REQUEST,"Set OID_NDIS_SMSC_SEND_E2P_COMMAND\r\n");	
			{
				DWORD cmd=*(DWORD *)InformationBuffer;
			
				if (NICEepromCommand(Adapter, cmd) !=NIC_STATUS_SUCCESS)
			 	{
					Status = NDIS_STATUS_FAILURE;
	   			}
			}
			break;	

	case OID_NDIS_SMSC_WRITE_E2P:	
			SMSC_TRACE0(DBG_REQUEST,"Set OID_NDIS_SMSC_WRITE_E2P\r\n");
			{
				WORD Address=*(WORD *)InformationBuffer;			
				BYTE ValueByte=*(BYTE *)((WORD *)InformationBuffer+1);
				if (NICWriteEepromByte(Adapter,Address, &ValueByte) !=NIC_STATUS_SUCCESS)
			 	{
					Status = NDIS_STATUS_FAILURE;
	   			}
			}
			break;

	case OID_NDIS_SMSC_WRITE_E2P_BLOCK:
			SMSC_TRACE0(DBG_REQUEST,"Set OID_NDIS_SMSC_WRITE_E2P_BLOCK\r\n");
			{
				DWORD E2pSize = *(DWORD *)InformationBuffer;
				BYTE *p = (BYTE *)((DWORD *)InformationBuffer+1);
		 
				if ( NICEepromCommand(Adapter, ETH_E2P_CMD_EWEN_)!=NIC_STATUS_SUCCESS) {
			   	Status = NDIS_STATUS_FAILURE;
				break;
				}
			
				if (NICWriteEepromBlock(Adapter,0,E2pSize,	p ) !=NIC_STATUS_SUCCESS) {
					Status = NDIS_STATUS_FAILURE;
				}

				NICEepromCommand(Adapter, ETH_E2P_CMD_EWDS_);	

			}		
			break;
	case OID_NDIS_SMSC_SET_LINK_STATUS:
			SMSC_TRACE0(DBG_REQUEST,"Set OID_NDIS_SMSC_SET_LINK_STATUS\r\n");
			{
				DWORD LinkUp = (*(DWORD *)InformationBuffer) & 0x1;
				BOOLEAN LinkChanged = FALSE;
				if (!MP_TEST_FLAG(Adapter, fMP_DISCONNECTED) && !LinkUp)
				{
					MP_SET_FLAG(Adapter, fMP_DISCONNECTED);
					LinkChanged = TRUE;
				} else if (MP_TEST_FLAG(Adapter, fMP_DISCONNECTED) && LinkUp)
				{
					MP_CLEAR_FLAG(Adapter, fMP_DISCONNECTED);
					LinkChanged = TRUE;
				}
				if (LinkChanged)
				{
					Status = NICIndicateMediaStatus(Adapter);
					if (Status != NIC_STATUS_SUCCESS)
					{
						Status = NDIS_STATUS_FAILURE;
					}
				}
			}
			break;
	default:
			Status = NDIS_STATUS_INVALID_OID;
			break;

	}

	if(Status == NDIS_STATUS_PENDING)
	{
		if (MP_IS_READY(Adapter))// && InformationBufferLength)
		{

			Status = NICScheduleRequest(Adapter,
							NdisRequestSetInformation,
							Oid,
							InformationBuffer,
							InformationBufferLength,
							BytesRead,
							BytesNeeded
							);

			ASSERT(Status == NDIS_STATUS_SUCCESS);
			Status = NDIS_STATUS_PENDING;
		}
		else
		{

			SMSC_TRACE2(DBG_REQUEST, "Flag=  0x%x, BufferLength=0x%8lx \r\n",  Adapter->Flags,InformationBufferLength );
			Status = NDIS_STATUS_NOT_ACCEPTED;
		}
	}


Exit_MPSetInformation:
	SMSC_TRACE2(DBG_REQUEST,"<-- MPSetInformation 0x%08lx, Status = 0x%08x\r\n",  Oid, Status);

	NdisReleaseSpinLock(&Adapter->RequestLock);

	return(Status);
}

/*++
Routine Description:
	This routine will query the hardware and return
	the media status.

Arguments:
	IN PMP_ADAPTER Adapter - pointer to adapter block

Return Value:
	NdisMediaStateDisconnected or
	NdisMediaStateConnected

--*/
ULONG
NICGetMediaConnectStatus(
	PMP_ADAPTER Adapter
	)
{
	SMSC_TRACE0(DBG_REQUEST,"--->NICGetMediaConnectStatus\r\n");

	if (MP_TEST_FLAG(Adapter, fMP_DISCONNECTED))
	{
	SMSC_TRACE0(DBG_REQUEST,"NdisMediaStateDisconnected\r\n");

		return(NdisMediaStateDisconnected);
	}
	else
	{
	SMSC_TRACE0(DBG_REQUEST,"NdisMediaStateConnected\r\n");
		return(NdisMediaStateConnected);
	}
	SMSC_TRACE0(DBG_REQUEST,"<---NICGetMediaConnectStatus\r\n");
}


/*++
Routine Description:
		This routine will query the hardware and return
		the Link Speed.

Arguments:
		IN PMP_ADAPTER Adapter - pointer to adapter block

Return Value:
	Maximum speed in kbps

--*/
ULONG
NICGetLinkSpeed(
	PMP_ADAPTER Adapter
	)
{
	DWORD LinkSpeed;
	SMSC_TRACE0(DBG_REQUEST,"--->NICGetLinkSpeed\r\n");

	if (MP_TEST_FLAG(Adapter, fMP_DISCONNECTED))
	{
		
		LinkSpeed = 1000000;
	}
	else
	{
		if(Adapter->LinkSpeed == LINK_SPEED_100MBS)
		{
			LinkSpeed = 1000000;
		}
		else
		{
			LinkSpeed = 100000;
		}
	}
	SMSC_TRACE0(DBG_REQUEST,"<---NICGetLinkSpeed\r\n");

	return(LinkSpeed);
}


/*++
Routine Description:
	This routine will set up the adapter so that it accepts packets
	that match the specified packet filter.  The only filter bits
	that can truly be toggled are for broadcast and promiscuous

Arguments:
	IN PMP_ADAPTER Adapter - pointer to adapter block
	IN ULONG PacketFilter - the new packet filter

Return Value:
	NDIS_STATUS_SUCCESS
	NDIS_STATUS_NOT_SUPPORTED

--*/

NDIS_STATUS
NICSetPacketFilter(
	IN PMP_ADAPTER Adapter,
	IN ULONG PacketFilter)
{
	NDIS_STATUS 	 Status = NDIS_STATUS_SUCCESS;

	SMSC_TRACE0(DBG_REQUEST,"--> NICSetPacketFilter\n");

	{
		SMSC_TRACE0(DBG_REQUEST,"Request to set NDIS Filters:\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_ALL_FUNCTIONAL)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_ALL_FUNCTIONAL\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_ALL_MULTICAST)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_ALL_MULTICAST\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_ALL_LOCAL)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_ALL_LOCAL\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_BROADCAST)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_BROADCAST\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_DIRECTED)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_DIRECTED\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_FUNCTIONAL)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_FUNCTIONAL\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_GROUP)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_GROUP\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_MAC_FRAME)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_MAC_FRAME\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_SMT)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_SMT\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_MULTICAST)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_MULTICAST\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_PROMISCUOUS)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_PROMISCUOUS\r\n");
		if( PacketFilter & NDIS_PACKET_TYPE_SOURCE_ROUTING)
			SMSC_TRACE0(DBG_REQUEST,"  NDIS_PACKET_TYPE_SOURCE_ROUTING\r\n");
		if( !(PacketFilter & 0xFFFFFFFFUL))
			SMSC_TRACE0(DBG_REQUEST,"  NONE\r\n");
		if(
		   PacketFilter & ~(
						NDIS_PACKET_TYPE_ALL_FUNCTIONAL |
						NDIS_PACKET_TYPE_ALL_MULTICAST |
						NDIS_PACKET_TYPE_ALL_LOCAL |
						NDIS_PACKET_TYPE_BROADCAST |
						NDIS_PACKET_TYPE_DIRECTED |
						NDIS_PACKET_TYPE_FUNCTIONAL |
						NDIS_PACKET_TYPE_GROUP |
						NDIS_PACKET_TYPE_MAC_FRAME |
						NDIS_PACKET_TYPE_SMT |
						NDIS_PACKET_TYPE_MULTICAST |
						NDIS_PACKET_TYPE_PROMISCUOUS |
						NDIS_PACKET_TYPE_SOURCE_ROUTING
				   )
		  )
		SMSC_TRACE1(DBG_REQUEST,"UNKNOWN FILTERS = %08X", PacketFilter &\
						~(
							NDIS_PACKET_TYPE_ALL_FUNCTIONAL |
							NDIS_PACKET_TYPE_ALL_MULTICAST |
							NDIS_PACKET_TYPE_ALL_LOCAL |
							NDIS_PACKET_TYPE_BROADCAST |
							NDIS_PACKET_TYPE_DIRECTED |
							NDIS_PACKET_TYPE_FUNCTIONAL |
							NDIS_PACKET_TYPE_GROUP |
							NDIS_PACKET_TYPE_MAC_FRAME |
							NDIS_PACKET_TYPE_SMT |
							NDIS_PACKET_TYPE_MULTICAST |
							NDIS_PACKET_TYPE_PROMISCUOUS |
							NDIS_PACKET_TYPE_SOURCE_ROUTING
						)\
				);
		}

	// any bits not supported?
	if(PacketFilter & ~NIC_SUPPORTED_FILTERS)
	{
		Status = NDIS_STATUS_NOT_SUPPORTED;
		SMSC_WARNING0("filter NDIS_STATUS_NOT_SUPPORTED\r\n");
		goto Exit_NICSetPacketFilter;
	}

	if (MP_TEST_FLAG(Adapter,fMP_DEVICE_NOT_ATTACHED)) {
		Status = NDIS_STATUS_FAILURE;
		goto Exit_NICSetPacketFilter;
		}

	// any filtering changes?
	if(PacketFilter != Adapter->PacketFilter)
	{
		// This will schedule the actual change in hw registers
		SMSC_TRACE0(DBG_REQUEST,"filter changed\r\n");
		Status = NDIS_STATUS_PENDING;
	}

Exit_NICSetPacketFilter:

	SMSC_TRACE0(DBG_REQUEST,"<-- NICSetPacketFilter\r\n");

	return(Status);
}

/*++
Routine Description:
	This routine will set up the adapter for a specified multicast
	address list.

Arguments:
	IN PMP_ADAPTER Adapter - Pointer to adapter block
	InformationBuffer		- Buffer for information
	InformationBufferLength   Size of this buffer
	pBytesRead				  Specifies how much info is read
	BytesNeeded 			  In case the buffer is smaller than

Return Value:

	NDIS_STATUS_

--*/


NDIS_STATUS
NICSetMulticastList(
	IN PMP_ADAPTER									Adapter,
	__in_bcount(InformationBufferLength) IN PVOID	InformationBuffer,
	IN ULONG										InformationBufferLength,
	OUT PULONG										pBytesRead,
	OUT PULONG										pBytesNeeded
	)
{
	NDIS_STATUS 		   Status = NDIS_STATUS_SUCCESS;

	UNREFERENCED_PARAMETER(InformationBuffer);
 
	SMSC_TRACE0(DBG_MULTICAST,"--> NICSetMulticastList\r\n");

	//
	// Initialize.
	//
	*pBytesNeeded = 0;
	*pBytesRead = InformationBufferLength;

	if (InformationBufferLength % ETH_LENGTH_OF_ADDRESS)
	{
		Status = NDIS_STATUS_INVALID_LENGTH;
		SMSC_TRACE2(DBG_MULTICAST,"NICSetMulticastList received invalid length (%d) not multiplpe of %d\r\n", InformationBufferLength, ETH_LENGTH_OF_ADDRESS);
		goto Exit_NICSetMulticastList;
	}

	if (InformationBufferLength > (NIC_MAX_MCAST_LIST * ETH_LENGTH_OF_ADDRESS))
	{
		*pBytesNeeded = NIC_MAX_MCAST_LIST * ETH_LENGTH_OF_ADDRESS;
		SMSC_TRACE2(DBG_MULTICAST,"NICSetMulticastList received invalid length (%d) bigger than size of multicast list (%d)\r\n",
							InformationBufferLength,
							NIC_MAX_MCAST_LIST * ETH_LENGTH_OF_ADDRESS);
		Status = NDIS_STATUS_MULTICAST_FULL;
		goto Exit_NICSetMulticastList;
	}

	if (MP_TEST_FLAG(Adapter,fMP_DEVICE_NOT_ATTACHED)) {
		Status = NDIS_STATUS_FAILURE;
		goto Exit_NICSetMulticastList;
		}
	//
	// Schedule to program the hardware to add suport for these muticast addresses
	//
	Status = NDIS_STATUS_PENDING;

Exit_NICSetMulticastList:

	SMSC_TRACE1(DBG_MULTICAST,"<-- NICSetMulticastList,Status = 0x%08x\r\n",Status);

	return(Status);
}

/*++

Routine Description:

		Utility routine that schedules an NDIS request made on a Adapter. Since
		at most a single request can be pended on a Adapter, we use the pre-allocated 
		request structure embedded in the Adapter struct.

Arguments:


Return Value:

		NDIS_STATUS_SUCCESS if the request is sent down.

--*/

NDIS_STATUS
NICScheduleRequest(
	IN PMP_ADAPTER					Adapter,
	IN NDIS_REQUEST_TYPE			RequestType,
	IN NDIS_OID 					Oid,
	IN PVOID						InformationBuffer,
	IN ULONG						InformationBufferLength,
	OUT PULONG						BytesReadOrWritten,
	OUT PULONG						BytesNeeded
	)
{
	PWDM_NDIS_REQUEST	wdmNdisRequest = &Adapter->WdmNdisRequest;

	SMSC_TRACE0(DBG_REQUEST,"--> NICScheduleRequest\r\n");

	ASSERT (!MP_TEST_FLAG(Adapter, fMP_REQUEST_PENDING));

	wdmNdisRequest->Oid = Oid;
	wdmNdisRequest->RequestType = RequestType;
	wdmNdisRequest->InformationBuffer = InformationBuffer;
	wdmNdisRequest->InformationBufferLength = InformationBufferLength;
	wdmNdisRequest->BytesNeeded = BytesNeeded;
	wdmNdisRequest->BytesReadOrWritten = BytesReadOrWritten;

	MP_SET_FLAG(Adapter, fMP_REQUEST_PENDING);

	MP_INC_REF(Adapter);

	NdisScheduleWorkItem(&wdmNdisRequest->WorkItem);

	SMSC_TRACE0(DBG_REQUEST,"<-- NICScheduleRequest\r\n");

	return (NDIS_STATUS_SUCCESS);
}


/*++

Routine Description:
	Work Item callback routine that handles Oid requests

Arguments:
		PNDIS_WORK_ITEM WorkItem		pointer to WDM_NDIS_REQUEST struct
		PVOID Context					pointer to our Adapter.

Return Value:
	None

--*/

VOID
NICRequestWorkItemCallback(
   PNDIS_WORK_ITEM WorkItem,
	PVOID Context
	)
{
	PMP_ADAPTER Adapter = (PMP_ADAPTER)Context;
	PWDM_NDIS_REQUEST wdmNdisRequest = (PWDM_NDIS_REQUEST)WorkItem;
	NDIS_STATUS Status = NDIS_STATUS_SUCCESS;	// Default
	int index=0;
	int 			Count = 0;


	SMSC_TRACE0(DBG_REQUEST,"--> NICRequestWorkItemCallback \r\n");

	// If an error recovery workitem is running, wait for it to finish
	// for a max of 1 seconds.
	while (Adapter->IsUsbErrorRecoveryWorkItemQueued && (Count++ < NIC_1_SEC_TIMEOUT))
	{
		NdisMSleep(NDIS_SLEEP_1_MS);	//Wait 1ms before trying again.
	}
	if (Count == NIC_1_SEC_TIMEOUT)
	{
		SMSC_TRACE0(DBG_REQUEST,"Timeout waiting for error recovery completion during Request workitem. Leaving Request pending !!!\r\n");
		MP_CLEAR_FLAG(Adapter, fMP_REQUEST_PENDING);
		goto Exit_NICRequestWorkItemCallback;
	}

	// If a reset is in progress, do not complete this OID, but
	// leave it pending. Just put ourselves in a state in which
	// we can handle new requests though. Ndis will resubmit the
	// one we left pending after the reset is completed.

	NdisAcquireSpinLock(&Adapter->RequestLock);
	if (MP_TEST_FLAG(Adapter, fMP_RESET_IN_PROGRESS))
	{
		SMSC_TRACE0(DBG_REQUEST,"Reset in progress detected during Request workitem. Leaving Request pending !!!\r\n");
		MP_CLEAR_FLAG(Adapter, fMP_REQUEST_PENDING);
		MP_CLEAR_FLAG(Adapter, fMP_NEED_USB_RESET);
		NdisReleaseSpinLock(&Adapter->RequestLock);
		goto Exit_NICRequestWorkItemCallback;
	}

	NdisReleaseSpinLock(&Adapter->RequestLock);

	ASSERT(wdmNdisRequest == &Adapter->WdmNdisRequest);
	if (wdmNdisRequest->RequestType == NdisRequestSetInformation)
	{
		switch(wdmNdisRequest->Oid)
		{
			case OID_GEN_CURRENT_PACKET_FILTER:
				{
				ULONG ReceiveMask;
				ULONG PacketFilter = *((PULONG)wdmNdisRequest->InformationBuffer);
				ReceiveMask = 0;

				SMSC_TRACE0(DBG_REQUEST,"OID_GEN_CURRENT_PACKET_FILTER\r\n");
				if(!(PacketFilter & NDIS_PACKET_TYPE_DIRECTED))
					ReceiveMask |= REJECT_DIRECTED;

				if(PacketFilter & NDIS_PACKET_TYPE_ALL_MULTICAST)
					ReceiveMask |= ACCEPT_MULTI_PROM;

				if(PacketFilter & NDIS_PACKET_TYPE_BROADCAST)
					ReceiveMask |= ACCEPT_BROADCAST;

				if(PacketFilter & NDIS_PACKET_TYPE_MULTICAST)
					ReceiveMask |= ACCEPT_MULTICAST;

				if(PacketFilter & NDIS_PACKET_TYPE_PROMISCUOUS)
				{
					ReceiveMask |= PROMISCUOUS_MODE | ACCEPT_BROADCAST |
					ACCEPT_MULTI_PROM;
				}

				Status=NICSetReceiveMask(Adapter, ReceiveMask);

				// Save the new packet filter value
				Adapter->PacketFilter = PacketFilter;

				if (Status == NDIS_STATUS_SUCCESS)
					Status = NDIS_STATUS_SUCCESS;
				else
					Status = NDIS_STATUS_FAILURE;
				}
				break;

			case OID_802_3_MULTICAST_LIST:

				SMSC_TRACE0(DBG_REQUEST,"NICRequestWorkItemCallback, OID_802_3_MULTICAST_LIST\r\n");
				NdisZeroMemory(Adapter->MCList,
						NIC_MAX_MCAST_LIST * ETH_LENGTH_OF_ADDRESS);

				NdisMoveMemory(Adapter->MCList,
							   wdmNdisRequest->InformationBuffer,
							   wdmNdisRequest->InformationBufferLength);

				Adapter->ulMCListSize = wdmNdisRequest->InformationBufferLength / ETH_LENGTH_OF_ADDRESS;
				
				if (Adapter->ulMCListSize >0)
				{
					for(index = 0; (ULONG)index < Adapter->ulMCListSize; index++)
					{
					   SMSC_TRACE7(DBG_MULTICAST,"MC(%d) = %02x-%02x-%02x-%02x-%02x-%02x\r\n",
						   index,
						  Adapter->MCList[index][0],
						  Adapter->MCList[index][1],
						  Adapter->MCList[index][2],
						  Adapter->MCList[index][3],
						  Adapter->MCList[index][4],
						  Adapter->MCList[index][5]);
					 }
				}
				else
				{
					SMSC_TRACE0(DBG_MULTICAST,"Delete Multicast list. \r\n");
				}

				#ifndef DO_NOT_USE_MC_HASH_FILTER
				// Program the multicast table
				Status=NICSetHwMulticastTable(
										Adapter,
										Adapter->ulMCListSize,
										(BYTE *) Adapter->MCList
									  );
				#endif
				
				if (Status == NDIS_STATUS_SUCCESS)
					Status = NDIS_STATUS_SUCCESS;
				else
					Status = NDIS_STATUS_FAILURE;
				
				break;

			case OID_PNP_SET_POWER:
			{
				BYTE PowerState;
				SMSC_TRACE0(DBG_REQUEST,"Changing to Power State D");

				switch (*((PNDIS_DEVICE_POWER_STATE)wdmNdisRequest->InformationBuffer))
				{
					case NdisDeviceStateD0:
						SMSC_TRACE0(DBG_POWER,"0\r\n");
						PowerState = 0;
						break;
					case NdisDeviceStateD1:
						SMSC_TRACE0(DBG_POWER,"1\r\n");
						PowerState = 1;
						break;
					case NdisDeviceStateD2:
						SMSC_TRACE0(DBG_POWER,"2\r\n");
						PowerState = 2;
						break;
					case NdisDeviceStateD3:
						SMSC_TRACE0(DBG_POWER,"3\r\n");
						PowerState = 3;
						break;
					default:
						SMSC_TRACE0(DBG_POWER,"? - Unknown request - Default to D0\r\n");
						PowerState = 0;
						break;
				}

				// Make sure the port is alive and well and we can set the Power state
				// Try three times.
				Count = 0;
				while (Count++ < 3)
				{
					if (NICSetPowerState(Adapter, PowerState) == NIC_STATUS_SUCCESS)
					{
						break;
					}
					else
					{
						SMSC_TRACE0(DBG_POWER,"NICSetPowerState failed !!!\r\n");
					}

				}

				// We could not get it done in three attempts, too bad...
				if (Count > 3)
				{
					// We are either unplugged or unrecoverable, get out.
					SMSC_TRACE0(DBG_POWER,"Failed to Change Power State 3 times. Returning failure to NDIS !!!\r\n");
					Status = NDIS_STATUS_FAILURE;
					
				}
				else
				{
					Status = NDIS_STATUS_SUCCESS;
				}

				break;
			}
			default:
				Status = NDIS_STATUS_INVALID_OID;
				break;
		}
	}
	else if (wdmNdisRequest->RequestType == NdisRequestQueryInformation)
	{
		SMSC_TRACE0(DBG_POWER,"NdisRequestQueryInformation\r\n");
	}
	else
	{
		SMSC_TRACE1(DBG_POWER,"NdisRequest Unknown (0x%08lx)\r\n", wdmNdisRequest->RequestType);
	}

	// Clear before the completion, to allow a new Oid scheduling from within
	// the context of the InformationComplete call.
	MP_CLEAR_FLAG(Adapter, fMP_REQUEST_PENDING);

	if(wdmNdisRequest->RequestType == NdisRequestSetInformation)
	{
		NdisMSetInformationComplete(Adapter->AdapterHandle, Status);
	}
	else
	{
		NdisMQueryInformationComplete(Adapter->AdapterHandle, Status);
	}


Exit_NICRequestWorkItemCallback:
	MP_DEC_REF(Adapter);

	// Did the USB recovery request that we schedule it's workitem ?
	NdisAcquireSpinLock(&Adapter->RequestLock);
	if (MP_TEST_FLAG(Adapter, fMP_NEED_USB_RESET))
	{
		// If we are not in D0, no need for the recovery workitem 
		//	 since USB has been turned off anyway
		if (Adapter->CurrentPowerState == 0)
		{
			// If device still present and workitem not already scheduled queue recovery.
			if ((!MP_TEST_FLAG(Adapter, fMP_SURPRISE_REMOVED)) &&
				(InterlockedExchange(&Adapter->IsUsbErrorRecoveryWorkItemQueued, TRUE) == FALSE)) 
			{
				Adapter->nUsbErrorRecoveryWorkItemScheduled++;
				MP_INC_REF(Adapter);				   
				NdisScheduleWorkItem(&Adapter->UsbErrorRecoveryWorkItem);
			}
			MP_CLEAR_FLAG(Adapter, fMP_NEED_USB_RESET);
		}
	}
	NdisReleaseSpinLock(&Adapter->RequestLock); 
	

	SMSC_TRACE1(DBG_REQUEST,"<-- NICRequestWorkItemCallback: %x\r\n", Status);

	return;
}


/*++

Routine Description:
	This routine gets a WOL pattern from the available pool and marks it as
	reserved.

Arguments:
		IN PMP_ADAPTER Adapter - pointer to adapter block

Return Value:
	Pattern #

--*/

BYTE
NICGetWOLPattern(
	IN PMP_ADAPTER Adapter
	)
{
	BYTE	i;

	SMSC_TRACE1(DBG_POWER,"Searching for an available WOLPattern. WolPatternEnabledMask = %08lx\r\n", Adapter->WOLPatternEnabledMask);

	for (i=0;i<Adapter->NumOfWolPatterns;i++)
	{
	 if( !(Adapter->WOLPatternEnabledMask & (1<<i)) )
	 {
		Adapter->WOLPatternEnabledMask |= (1<<i);
		Adapter->WOLPatternsInUse++;
		break;
	 }
	}

	SMSC_TRACE1(DBG_POWER,"WolPatternEnabledMask is now = %08lx\r\n", Adapter->WOLPatternEnabledMask);

	return(i);
}


/*++

Routine Description:
	This routine adds a WOL sample specified.

Arguments:
	IN PMP_ADAPTER Adapter - pointer to adapter block

Return Value:
	NIC_STATUS

--*/

NDIS_STATUS
NICAddWOLSample(
	IN PMP_ADAPTER Adapter,
	PLM_WOL_SAMPLE_STRUCT pLMWolSample
	)
{
	NDIS_STATUS Status;

	BYTE *pMaskSP, *pSampleSP, *pMask, *pSample;

	BYTE PatternNumber;
	WORD i, First, Last, SampleOffset, Count;

	DWORD SampleSize;

	SMSC_TRACE0(DBG_POWER,"--> NICAddWOLSample\r\n");

	pSample =		pLMWolSample->pWOLSample;
	SampleSize =	pLMWolSample->WOLSampleSize;
	pMask = 		pLMWolSample->pWOLMask;
	First = 		pLMWolSample->FirstEnabledByte;
	Last =			pLMWolSample->LastEnabledByte;

	
	// The pattern starts at the offset of the first enabled bit in the mask. The
	// mask is scanned and recreated taking into account the offset shifting in the
	// WOL mask scratchpad area. The enabled bytes in the bit mask are also copied
	// to the WOL sample scratchpad area. Once we complete the <SizeOfWolPattern>
	// bytes, if we found at least one bit mask set, we program the chip to scan
	// the WOL pattern.

	PatternNumber = pLMWolSample->HeadPattern;

	SMSC_TRACE1(DBG_POWER,"Programming pattern #%d\r\n", PatternNumber);

	// Initialize pointers and variables

	Count = 0;
	pMaskSP = Adapter->pWolMaskScratchpad;
	pSampleSP = Adapter->pWolSampleScratchpad;

	for(i=0;i<SIZE_OF_WOL_PATTERN;i++)
	{
		SampleOffset = (WORD) (First + i);

		// If the byte exists in the original sample and the bit is set in the
		// original mask then set the bit in the new mask and copy the byte, otherwise
		// reset the bit in the mask

		if (
			(SampleOffset >= SampleSize) ||
			(!( pMask[SampleOffset/8] & (1 << (SampleOffset % 8)) ))
		   )
		{
			pMaskSP[i/8] &= ~(1 << (i % 8));
		}
		else
		{
			pMaskSP[i/8] |= (1 << (i % 8));
			pSampleSP[Count++] = pSample[SampleOffset];
		}
	}

	SMSC_TRACE2(DBG_POWER,"Offset[%d] = %04x\r\n", PatternNumber, First);

	#ifdef SMSC_DBG
	if (g_DebugMode & DBG_REQUEST) {
		SMSC_TRACE1(DBG_POWER,"Mask Regs [%d] = ", PatternNumber);
		for (i=0;i<SIZE_OF_WOL_PATTERN_IN_DW;i++)
			RETAILMSG(1,(TEXT("%08lx "), (DWORD) *((DWORD *) (pMaskSP+i*4)) ));
		RETAILMSG(1,(TEXT("\r\n")));

		SMSC_TRACE2(DBG_POWER,"Bytes of data (%d) for CRC calculation [%d]:\r\n   ", Count, PatternNumber);
		for (i=0;i<Count;i++)
		{
			if ((i % 16) == 0)
				RETAILMSG(1,(TEXT("\r\n")));
			RETAILMSG(1,(TEXT("%02x "), *(pSampleSP+i)) );
		}
		RETAILMSG(1,(TEXT("\r\n")));
	}
	#endif

	// If there was at least one byte enabled in the mask program it in the chip

	if(Count)
	{
		WORD crc;

		// Save the mask
		for (i=0; i<SIZE_OF_WOL_PATTERN_IN_DW; i++)
		{
			Adapter->dwWUFF[PatternNumber * SIZE_OF_WOL_PATTERN_IN_DW + i] = * ((PDWORD) pMaskSP);
			pMaskSP += 4;
		}

		crc = NicCalculateCrc16(pSampleSP, Count, FALSE);

		SMSC_TRACE2(DBG_POWER,"crc16[%d] = %04lx\r\n",PatternNumber,crc);
		if (Adapter->NumOfWolPatterns == 4)
		{
			Adapter->dwWUFF[WUFF_4_CMD] |= ((((WUFF_CMD_ANY_) | WUFF_CMD_EN_) << (PatternNumber * 8)));

			// Set the offset from the beginning of the packet where to start looking 
			Adapter->dwWUFF[WUFF_4_OFF] |= (First << (PatternNumber * 8));

			// Set the Crc
			Adapter->dwWUFF[WUFF_4_CRC_01 + (PatternNumber / 2)] |= (crc << ((PatternNumber % 2) * 16));
		}
		else
		{
			Adapter->dwWUFF[WUFF_8_CMD03 + (PatternNumber / 4)] |= ((((WUFF_CMD_ANY_) | WUFF_CMD_EN_) << ((PatternNumber % 4) * 8)));

			// Set the offset from the beginning of the packet where to start looking 
			Adapter->dwWUFF[WUFF_8_OFF03 + (PatternNumber / 4)] |= (First << ((PatternNumber % 4) * 8));

			// Set the Crc
			Adapter->dwWUFF[WUFF_8_CRC_01 + (PatternNumber / 2)] |= (crc << ((PatternNumber % 2) * 16));
		}

		// Set the flag
		pLMWolSample->Enabled = TRUE;
	}
	else
	{
		SMSC_TRACE0(DBG_POWER,"Empty pattern, not adding.\r\n");
	}

#if 0	
	if (Adapter->NumOfWolPatterns == 4)
	{
	SMSC_TRACE0(DBG_POWER,"WUFF saved in mem for later register programming:\r\n");
		for (i=0;i<=WUFF_4_CRC_23;i++)
		{
			SMSC_TRACE2(DBG_POWER,"WUFF[%02d] = %08lx\r\n", i, Adapter->dwWUFF[i]);
			// The actual WUFF register will be programmed accordingly at the time we go to Dx (x>0)
		}
	}
	else
	{
		SMSC_TRACE0(DBG_POWER,"WUFF saved in mem for later register programming:\r\n");
		for (i=0;i<=WUFF_8_CRC_67;i++)
	{
		SMSC_TRACE2(DBG_POWER,"WUFF[%02d] = %08lx\r\n", i, Adapter->dwWUFF[i]);
		// The actual WUFF register will be programmed accordingly at the time we go to Dx (x>0)
	}
	}
#endif

	Status = NDIS_STATUS_SUCCESS;

	SMSC_TRACE0(DBG_POWER,"<-- NICAddWOLSample\r\n");

	return Status;
}

/*++

Routine Description:
	This routine parses the loaded samples looking for the one requested

Arguments:
		IN PMP_ADAPTER Adapter - pointer to adapter block
	PLM_WOL_SAMPLE_STRUCT - pointer to the sample to look for
							(only mask, sample data and their sizes,
							 are loaded on entry - the Head Pattern is
							 not loaded)

Return Value:
	PLM_WOL_SAMPLE_STRUCT - pointer to the real sample with the head pattern loaded,
							or NULL if not found.

--*/

PLM_WOL_SAMPLE_STRUCT
NICFindWolSample(
	IN PMP_ADAPTER Adapter,
	PLM_WOL_SAMPLE_STRUCT pLMWolSample
	)
{
	WORD i, j;
	BOOLEAN Found = TRUE;
	PLM_WOL_SAMPLE_STRUCT pLMWolMatcher;

	for (i=0;i<Adapter->NumOfWolPatterns;i++)
	{
		Found = TRUE;
		pLMWolMatcher = (PLM_WOL_SAMPLE_STRUCT) (Adapter->NicWolMemBlock + (i * sizeof(LM_WOL_SAMPLE_STRUCT)));

		SMSC_TRACE1(DBG_POWER,"Checking for match with sample #%d ... ", i);

		if (
			(!pLMWolMatcher->Enabled) ||
			(pLMWolMatcher->WOLSampleSize != pLMWolSample->WOLSampleSize) ||
			(pLMWolMatcher->WOLMaskSize != pLMWolSample->WOLMaskSize)
		   )
		{
			Found = FALSE;
			goto Next_Pattern;
		}

		for(j=0;j<pLMWolSample->WOLMaskSize;j++)
		{
			if(pLMWolMatcher->pWOLMask[j] != pLMWolSample->pWOLMask[j])
			{
				Found = FALSE;
				break;
			}
		}

		if(!Found)
			goto Next_Pattern;

		for(j=0;j<pLMWolSample->WOLSampleSize;j++)
		{
			if(pLMWolMatcher->pWOLSample[j] != pLMWolSample->pWOLSample[j])
			{
				Found = FALSE;
				break;
			}
		}

Next_Pattern:
		if(Found)
		{
			SMSC_TRACE0(DBG_POWER,"MATCHED!\r\n");
			return (pLMWolMatcher);
		}
		else
		{
			SMSC_TRACE0(DBG_POWER,"NO match.\r\n");
		}
	}

	SMSC_TRACE0(DBG_POWER,"Could not find any matching sample\r\n");

	return (NULL);
}

/*++

Routine Description:
	This routine removes the WOL sample specified.

Arguments:
		IN PMP_ADAPTER Adapter - pointer to adapter block

Return Value:
	NIC_STATUS

--*/
void
NICRemoveWOLSample(
	IN PMP_ADAPTER Adapter,
	PLM_WOL_SAMPLE_STRUCT pLMWolSample
	)
{
	BYTE PatternNumber = pLMWolSample->HeadPattern;
	int i;

	SMSC_TRACE2(DBG_POWER,"--> NICRemoveWOLSample %p. Releasing Pattern # %d\r\n", pLMWolSample, PatternNumber);

	for (i=0; i<SIZE_OF_WOL_PATTERN_IN_DW; i++)
	{
		Adapter->dwWUFF[PatternNumber * SIZE_OF_WOL_PATTERN_IN_DW + i] = 0;
	}
	if (Adapter->NumOfWolPatterns == 4)
	{
		Adapter->dwWUFF[WUFF_4_CMD] &= (~((DWORD)(0xFF << (PatternNumber * 8))));
		Adapter->dwWUFF[WUFF_4_OFF] &= (~((DWORD)(0xFF << (PatternNumber * 8))));
		Adapter->dwWUFF[WUFF_4_CRC_01 + (PatternNumber / 2)] &= (~((DWORD)(0xFFFF << ((PatternNumber % 2) * 16))));
	}
	else
	{
		Adapter->dwWUFF[WUFF_8_CMD03 + (PatternNumber / 4)] &= (~((DWORD)(0xFF << ((PatternNumber % 4) * 8))));
		Adapter->dwWUFF[WUFF_8_OFF03 + (PatternNumber / 4)] &= (~((DWORD)(0xFF << ((PatternNumber % 4) * 8))));
		Adapter->dwWUFF[WUFF_8_CRC_01 + (PatternNumber / 2)] &= (~((DWORD)(0xFFFF << ((PatternNumber % 2) * 16))));
	}

// #if 0
	SMSC_TRACE0(DBG_POWER,"WUFF saved in mem for later register programming:\r\n");
	if (Adapter->NumOfWolPatterns == 4)
	{
		for (i=0;i<=WUFF_4_CRC_23;i++)
	{
		SMSC_TRACE2(DBG_POWER,"WUFF[%02d] = %08lx\r\n", i, Adapter->dwWUFF[i]);
		// The actual WUFF register will be programmed accordingly at the time we go to Dx (x>0)
	}
	}
	else
	{
		for (i=0;i<=WUFF_8_CRC_67;i++)
		{
			SMSC_TRACE2(DBG_POWER,"WUFF[%02d] = %08lx\r\n", i, Adapter->dwWUFF[i]);
			// The actual WUFF register will be programmed accordingly at the time we go to Dx (x>0)
		}
	}
//#endif 

	Adapter->WOLPatternEnabledMask &= (~(1<<PatternNumber));
	Adapter->WOLPatternsInUse--;

	// Clear the flag
	pLMWolSample->Enabled = FALSE;

	SMSC_TRACE0(DBG_POWER,"<-- NICRemoveWOLSample\r\n");
}

/*++

Routine Description:
	This routine calculates a 16 bit CRC on a sequence of bytes

Arguments:
	BYTE *bpData		points to an array of bytes to calculate the CRC on.
	DWORD dwLen			the size (in bytes) of the array
	DWORD fBitReverse	True if need to reverse result.

Return Value:
	16 bit CRC

--*/
WORD
NicCalculateCrc16(
	const BYTE * bpData,
	const DWORD dwLen,
	const DWORD fBitReverse
	)
{
	const WORD wCrc16Poly = 0x8005U;	// s/b international standard for CRC-16
										// x^16 + x^15 + x^2 + 1
	WORD i, j, bit;
	WORD wCrc = 0xFFFFU;
	WORD wMsb;
	BYTE bCurrentByte;
	WORD wNumOfBits = 16U;
	WORD wCrcOut=0;

	wNumOfBits = wNumOfBits; // to avoid lint warning

	for (i=0; i<(WORD)dwLen; i++)
	{
		bCurrentByte = *bpData++;

		for (bit=(WORD)0U; bit<(WORD)8U; bit++)
		{
			wMsb = wCrc >> 15;
			wCrc <<= 1;

			if (wMsb ^ (WORD)(bCurrentByte & 1))
			{
				wCrc ^= wCrc16Poly;
				wCrc |= (WORD)0x0001U;
			}
			bCurrentByte >>= 1;
		}
	}

	//bit reverse if needed
	// but the standard CRC-16 seems to require this.
	if (fBitReverse)
	{
		j = 1;
		for (i=(WORD)(1<<(wNumOfBits-(WORD)1U)); i; i = i>>1) {
			if (wCrc & i)
			{
				wCrcOut |= j;
			}
			j <<= 1;
		}
		wCrc = wCrcOut;
	}

	return wCrc;
}


PUCHAR
DbgGetOidName(
	ULONG oid
	)
{
	PCHAR oidName;

	switch (oid){

		#undef MAKECASE
		#define MAKECASE(oidx) case oidx: oidName = #oidx; break;

		MAKECASE(OID_GEN_SUPPORTED_LIST)
		MAKECASE(OID_GEN_HARDWARE_STATUS)
		MAKECASE(OID_GEN_MEDIA_SUPPORTED)
		MAKECASE(OID_GEN_MEDIA_IN_USE)
		MAKECASE(OID_GEN_MAXIMUM_LOOKAHEAD)
		MAKECASE(OID_GEN_MAXIMUM_FRAME_SIZE)
		MAKECASE(OID_GEN_LINK_SPEED)
		MAKECASE(OID_GEN_TRANSMIT_BUFFER_SPACE)
		MAKECASE(OID_GEN_RECEIVE_BUFFER_SPACE)
		MAKECASE(OID_GEN_TRANSMIT_BLOCK_SIZE)
		MAKECASE(OID_GEN_RECEIVE_BLOCK_SIZE)
		MAKECASE(OID_GEN_VENDOR_ID)
		MAKECASE(OID_GEN_VENDOR_DESCRIPTION)
		MAKECASE(OID_GEN_CURRENT_PACKET_FILTER)
		MAKECASE(OID_GEN_CURRENT_LOOKAHEAD)
		MAKECASE(OID_GEN_DRIVER_VERSION)
		MAKECASE(OID_GEN_MAXIMUM_TOTAL_SIZE)
		MAKECASE(OID_GEN_PROTOCOL_OPTIONS)
		MAKECASE(OID_GEN_MAC_OPTIONS)
		MAKECASE(OID_GEN_MEDIA_CONNECT_STATUS)
		MAKECASE(OID_GEN_MAXIMUM_SEND_PACKETS)
		MAKECASE(OID_GEN_VENDOR_DRIVER_VERSION)
		MAKECASE(OID_GEN_SUPPORTED_GUIDS)
		MAKECASE(OID_GEN_NETWORK_LAYER_ADDRESSES)
		MAKECASE(OID_GEN_TRANSPORT_HEADER_OFFSET)
		MAKECASE(OID_GEN_MEDIA_CAPABILITIES)
		MAKECASE(OID_GEN_PHYSICAL_MEDIUM)
		MAKECASE(OID_GEN_XMIT_OK)
		MAKECASE(OID_GEN_RCV_OK)
		MAKECASE(OID_GEN_XMIT_ERROR)
		MAKECASE(OID_GEN_RCV_ERROR)
		MAKECASE(OID_GEN_RCV_NO_BUFFER)
		MAKECASE(OID_GEN_DIRECTED_BYTES_XMIT)
		MAKECASE(OID_GEN_DIRECTED_FRAMES_XMIT)
		MAKECASE(OID_GEN_MULTICAST_BYTES_XMIT)
		MAKECASE(OID_GEN_MULTICAST_FRAMES_XMIT)
		MAKECASE(OID_GEN_BROADCAST_BYTES_XMIT)
		MAKECASE(OID_GEN_BROADCAST_FRAMES_XMIT)
		MAKECASE(OID_GEN_DIRECTED_BYTES_RCV)
		MAKECASE(OID_GEN_DIRECTED_FRAMES_RCV)
		MAKECASE(OID_GEN_MULTICAST_BYTES_RCV)
		MAKECASE(OID_GEN_MULTICAST_FRAMES_RCV)
		MAKECASE(OID_GEN_BROADCAST_BYTES_RCV)
		MAKECASE(OID_GEN_BROADCAST_FRAMES_RCV)
		MAKECASE(OID_GEN_RCV_CRC_ERROR)
		MAKECASE(OID_GEN_TRANSMIT_QUEUE_LENGTH)
		MAKECASE(OID_GEN_GET_TIME_CAPS)
		MAKECASE(OID_GEN_GET_NETCARD_TIME)
		MAKECASE(OID_GEN_NETCARD_LOAD)
		MAKECASE(OID_GEN_DEVICE_PROFILE)
		MAKECASE(OID_GEN_INIT_TIME_MS)
		MAKECASE(OID_GEN_RESET_COUNTS)
		MAKECASE(OID_GEN_MEDIA_SENSE_COUNTS)
		MAKECASE(OID_PNP_CAPABILITIES)
		MAKECASE(OID_PNP_SET_POWER)
		MAKECASE(OID_PNP_QUERY_POWER)
		MAKECASE(OID_PNP_ADD_WAKE_UP_PATTERN)
		MAKECASE(OID_PNP_REMOVE_WAKE_UP_PATTERN)
		MAKECASE(OID_PNP_ENABLE_WAKE_UP)
		MAKECASE(OID_802_3_PERMANENT_ADDRESS)
		MAKECASE(OID_802_3_CURRENT_ADDRESS)
		MAKECASE(OID_802_3_MULTICAST_LIST)
		MAKECASE(OID_802_3_MAXIMUM_LIST_SIZE)
		MAKECASE(OID_802_3_MAC_OPTIONS)
		MAKECASE(OID_802_3_RCV_ERROR_ALIGNMENT)
		MAKECASE(OID_802_3_XMIT_ONE_COLLISION)
		MAKECASE(OID_802_3_XMIT_MORE_COLLISIONS)
		MAKECASE(OID_802_3_XMIT_DEFERRED)
		MAKECASE(OID_802_3_XMIT_MAX_COLLISIONS)
		MAKECASE(OID_802_3_RCV_OVERRUN)
		MAKECASE(OID_802_3_XMIT_UNDERRUN)
		MAKECASE(OID_802_3_XMIT_HEARTBEAT_FAILURE)
		MAKECASE(OID_802_3_XMIT_TIMES_CRS_LOST)
		MAKECASE(OID_802_3_XMIT_LATE_COLLISIONS)

		default:
			oidName = "<** UNKNOWN OID **>";
			break;
	}

	return oidName;
}


