/*++
	Copyright (c) 2008-2009 SMSC. All rights reserved.

	The information contained in this file is confidential and proprietary to
	Standard Microsystems Corporation.	No part of this file may be reproduced
	or distributed, in any form or by any means for any purpose, without the
	express written permission of Standard Microsystems Corporation.

Module Name:
	SMSC9500.h
Abstract:
	This module contains structure definitons and function prototypes.
--*/

#if !defined( _SMSC9500_ )
#define _SMSC9500_

//*****************************************************************************
// I N C L U D E S
//*****************************************************************************
#include <ndis.h>
#include <windows.h>
#include <usbdi.h>
#include "usbclient.h"
#include "lan9500.h"

#define BIDIMENSIONAL_SEND_LIST

/*
#ifndef USE_WORK_ITEM_RESUBMIT_RCB
#define USE_WORK_ITEM_RESUBMIT_RCB
#endif
*/

/*
#ifndef USE_INT_PIPE
#define USE_INT_PIPE
#endif
*/

#ifndef USE_WORK_ITEM_NO_OUT
#define USE_WORK_ITEM_NO_OUT
#endif

/*
#ifndef USE_THREAD
#define USE_THREAD
#endif
*/

//*****************************************************************************
// D E F I N E S
//*****************************************************************************

#define Max_Instances	9

//Note: Num_of_Instances_supported  should NOT greater than Max_Instances
#define Num_of_Instances_supported 5   //This number should NOT greater than Max_Instances

//
// Is the context pointer valid
//
#define VALID_CONTEXT( p ) \
   ( (p != NULL) && SMSC_SIG == p->Sig )

//
// Can the device accept any I/O requests
//
#define ACCEPT_IO( p ) \
   ( VALID_CONTEXT( p ) && \
	 p->Dev_Flags.AcceptIo && \
	!(p->Dev_Flags.DeviceRemoved) )

#define SMSC9500_DEVICE_CLASS			0xFF
#define SMSC9500_DEVICE_SUBCLASS		0x00
#define SMSC9500_DEVICE_PROTOCOL		0x01

#define SMSC9500_DEVICE_VENDOR_ID		0x0424
#define SMSC9500_DEVICE_PRODUCT_ID		0x9500
#define SMSC9500_DEVICE_RELEASE_NUMBER	0x0100

#define SMSC9500_INTERFACE_CLASS		0xFF
#define SMSC9500_INTERFACE_SUBCLASS 	0x00
#define SMSC9500_INTERFACE_PROTOCOL 	0xFF

// Vendor Requests
#define USB_REQUEST_TYPE_WRITE_REGISTER		0x40
#define USB_REQUEST_TYPE_READ_REGISTER		0xC0

#define USB_VENDOR_REQUEST_WRITE_REGISTER	0xA0
#define USB_VENDOR_REQUEST_READ_REGISTER	0xA1
#define USB_VENDOR_REQUEST_GET_STATS		0xA2

#if defined(NDIS50_MINIPORT)
	#define SMSC9500_NDIS_MAJOR_VERSION 	  5
	#define SMSC9500_NDIS_MINOR_VERSION 	  0
#elif defined(NDIS51_MINIPORT)
	#define SMSC9500_NDIS_MAJOR_VERSION 	  5
	#define SMSC9500_NDIS_MINOR_VERSION 	  1
#endif

#define DRIVER_NAME_SZ		TEXT("SMSC9500.DLL")
#define DEVICE_PREFIX		TEXT("LAN")

#define CLASS_NAME_SZ		TEXT("SMSC9500")
//#define Device_Instance  TEXT("SMSC95001")
static PWSTR  pDeviceInstance[Max_Instances]={L"SMSC95001",L"SMSC95002",L"SMSC95003",L"SMSC95004",L"SMSC95005",L"SMSC95006",L"SMSC95007",L"SMSC95008",L"SMSC95009"};
#define CLIENT_REGKEY_SZ	TEXT("\\Drivers\\USB\\ClientDrivers\\SMSC9500")
#define SMSC_SIG 'SMSC'
#define DRIVER_COMPANYNAME_STR	"SMSC"


#define	NDIS_ALLOC_FLAG		(0U) // Flag for NdisAllocMemory and NdisFreeMemory

#define SMSC9500_DRIVER_SETTINGS \
			sizeof(USB_DRIVER_SETTINGS),  \
			SMSC9500_DEVICE_VENDOR_ID,	 \
			SMSC9500_DEVICE_PRODUCT_ID,   \
			USB_NO_INFO,   \
			USB_NO_INFO,   \
			USB_NO_INFO,   \
			USB_NO_INFO,   \
			SMSC9500_INTERFACE_CLASS, \
			USB_NO_INFO,	\
			USB_NO_INFO

//*****************************************************************************
// T Y P E D E F S
//*****************************************************************************

//
// Device Flags
//
typedef struct _DEVICE_FLAGS {
	//
	// The device is initialized and ready to accept I/O
	//
	BOOLEAN   AcceptIo ;
	//
	// The device has been removed
	//
	BOOLEAN   DeviceRemoved;

	UCHAR	PipeClosed : 1;

} DEVICE_FLAGS, *PDEVICE_FLAGS;



typedef struct _PIPE {
	//
	// USB Pipe handle received from the stack
	//
	USB_PIPE hPipe;

	//
	// Endpoint's Address
	//
	UCHAR	 bIndex;

	//
	// Endpoint's wMaxPacketSize
	//
	USHORT wMaxPacketSize;

	HANDLE	 hEvent;

	BOOL PipeTransferAborted;

} PIPE, *PPIPE;

typedef struct _MP_ADAPTER
{	
	PWSTR   InstanceName;
	DWORD	InstanceIndex;
	LONG					RefCount;
//	NDIS_SPIN_LOCK			Lock;
	NDIS_EVENT				HaltEvent;
	HANDLE					PhyMutex;
	//
	// Keep track of various device objects.
	//
//	PDEVICE_OBJECT			Pdo;
//	PDEVICE_OBJECT			Fdo;
//	PDEVICE_OBJECT			NextDeviceObject;
//	PDEVICE_OBJECT			TargetDeviceObject;
	NDIS_HANDLE 			AdapterHandle;
	NDIS_HANDLE 			WrapperConfigurationContext;
	WCHAR					AdapterName[NIC_ADAPTER_NAME_SIZE];
	WCHAR					AdapterDesc[NIC_ADAPTER_NAME_SIZE];
	ULONG					Flags;
	UCHAR					PermanentNodeAddress[ETH_LENGTH_OF_ADDRESS];
	UCHAR					CurrentNodeAddress[ETH_LENGTH_OF_ADDRESS];
	DWORD                   GetNodeAddrFromEep;
    DWORD                   GetNodeAddrFromEepRam;
	BOOL					MPInitializeFail;
	BOOL					MPInitialized;

	DWORD	AdapterNumber;
	DWORD	IdReg;
	DWORD	DevIdReg;
#define DISABLED		0
#define FC_SYMM 		1
#define FC_ASYM_GEN 	2
#define FC_ASYM_RESP	3
	DWORD	FlowControl;
	DWORD	AFCCfgReg;
	DWORD	FCPause;

	DWORD	PowerManagement;
	DWORD	WolPatternMatchSettings;
	DWORD	WolMagicPacketSettings;
	DWORD	WoLinkChgSettings;
#define ENABLED 		1
#define OS_CONTROLLED	1
#define FORCED			2

#define WAKE_CLEAR_DISABLED 	0
#define WAKE_CLEAR_EN			1
#define WAKE_CLEAR_STS			2
#define WAKE_CLEAR_EN_AND_STS	3

	DWORD	WakeAutoClear;
	DWORD	CurrentPowerState;
	BOOLEAN SetFeatureRemoteWake;

	ULONG	NicWolMemSize;
	PUCHAR	NicWolMemBlock;
	BYTE	*pWolMaskScratchpad;
	BYTE	*pWolSampleScratchpad;
	BYTE	WOLPatternsInUse;
	DWORD	WOLPatternEnabledMask;
	BOOLEAN UsbDeviceSupportsWakes;
	DWORD	NumOfWolPatterns;

	DWORD	VlanID;
	DWORD	PriorityVlanTag;

	BOOLEAN SubmitRCBFail;

#define NIC_FEATURE_POWER_STATES	0x00000001
#define NIC_FEATURE_WAKE_ON_LAN 	0x00000002
#define NIC_FEATURE_MAGIC_PACKET	0x00000004
#define NIC_FEATURE_LINK_CHANGE 	0x00000008
	DWORD	CurrentWakeEvents;

	DWORD	dwWUFF[40];

	// Timers
#define LINK_CHECK_INTERVAL 1000		// 1s
	NDIS_MINIPORT_TIMER LinkCheckTimer;
	NDIS_SPIN_LOCK			LCSpinlock;
	NDIS_EVENT LinkCheckCalledEvent;

	NDIS_SPIN_LOCK			RequestLock;
	DWORD	MediaType;
	DWORD	AdvertisedCapabilites;
	DWORD	PHYAddress; // For external PHY support, this is the value read
						//	from the Registry

#define PHY_BOOST_NORMAL	0
#define PHY_BOOST_PLUS_4	1
#define PHY_BOOST_PLUS_8	2
#define PHY_BOOST_PLUS_12	3
	DWORD	PHYDriveStrength; // PHY drive strength in THERA 130

	DWORD	PHY_OUI;
	BYTE	PHY_Model;
	BYTE	PHY_Revision;

#define PHY_INT_SRC_FLAGS_ROLH  0
#define PHY_INT_SRC_FLAGS_WC    1
    DWORD   PhyIntSrcFlagsClr; 
    
#define PHY_HOOKUP_INTERNAL     0
#define PHY_HOOKUP_EXTERNAL     1
#define PHY_HOOKUP_AUTODETECT   2
    DWORD   PhyHookup;

#define MDIO_DISABLE_NONE		0x0
#define MDIO_DISABLE_INTERNAL	0x1
#define MDIO_DISABLE_IOCTL		0x2
#define MDIO_DISABLE_ALL		0x3
	DWORD   MdioDisable;

#define DEFAULT_LINK_DOWN_ALL			0x0
#define DEFAULT_LINK_UP_AFTER_INIT		0x1
#define DEFAULT_LINK_UP_AFTER_RESET		0x2
#define DEFAULT_LINK_UP_AFTER_PM		0x4
#define DEFAULT_LINK_UP_ALL				0x7
	DWORD	DefaultLinkStatus;

	//To be used when connected to switch to include other phys link status
	DWORD MdioPhyAddressStart; //Start phy to check
	DWORD MdioPhyAddressCount; //Last number of phy to check

#define LINK_WAKE_MODE_OFF		0x00000000
#define LINK_WAKE_MODE_DEFAULT	(ETH_GPIO_WAKE_PHY_LINKUP_EN_ | PHY_THERA_INT_NWAY_DONE)
#define LINK_WAKE_MODE_ALL		(ETH_GPIO_WAKE_PHY_LINKUP_EN_ | 0x000000FFUL)
    DWORD   LinkWakeModes;
    DWORD   SDReconnectGpio;
#define DRIVEN_LOW  0
#define DRIVEN_HIGH 1
    DWORD   SDReconnectGpioPol;
	
	DWORD	AutoMdix;
	DWORD	SaveAutoMdix;
#define AMDIX_MODE_AUTO       0
#define AMDIX_MODE_ON_802_3   1
#define AMDIX_MODE_ON_EXTEND  2
    DWORD   AutoMdixMode;
	DWORD   DetachOnLinkDownMode;
	DWORD	SleepPhyOnLinkDown;
	BOOLEAN PhyLinkPowerDown;
	DWORD	NoBulkInStopOnLinkDown;
#define DOLD_DISABLED       0
#define DOLD_ENABLED        1  // Always enable Smartdetach

	DWORD EDPDConfig;  
	DWORD EDPDNetDetachTimer;
	DWORD EDPDNetDetachTimeout;

	BYTE PerformingNWAY;
	BYTE PerformingNWAYTimerCounter;

	DWORD ReceiveMask;
#ifndef DO_NOT_USE_MC_HASH_FILTER
	DWORD MulticastHashTable[2];
#endif
	DWORD LinkSpeed;
	DWORD LinkDuplex;

	BOOLEAN Timercancelled;

	// Link/Activity LED configuration
	DWORD					LinkActLedCfg;
	
	// Software control GPIO to display Link up/down
#define 	LED_ON			1
#define 	LED_OFF 		0
	DWORD					LinkLedOnGpio;
	DWORD					LinkLedOnGpioBufType;
	DWORD					LinkLedOnGpioPolarity;

	ULONG SecondsElapsedAfterMPInitializeExit;
	ULONG SecondsElapsedAfterMPResetExit;

	WDM_NDIS_REQUEST		WdmNdisRequest;


	BOOLEAN 				UsbSpeedIsHigh;
	ULONG Sig;

	//
	// USB handle to our device
	//
	HANDLE	 hUsbDevice;
	ULONG	 Index;

	//
	// USBD Function table
	//
	LPCUSB_FUNCS   UsbFuncs;

	//
	// Fields from USB_INTERFACE that we need
	//
	LPCUSB_INTERFACE pUsbInterface;

	//
	// USB Configutation Index
	//
	USHORT ConfigIndex;

	//
	// Bulk OUT Pipe
	//
	PIPE BulkOut;

	//
	// Bulk IN Pipe
	//
	PIPE BulkIn;

	//
	// Interrupt Pipe
	//
	PIPE  Interrupt;
   
	//
	// FLAGS
	//
	DEVICE_FLAGS Dev_Flags;
	
	HANDLE						hNdisStartUpThread;
	BOOL halting;
	BOOLEAN NdisInitialized;
	
	//
	// Variables to track resources for the send operation
	//
	LIST_ENTRY				SendPktWaitList;
	LIST_ENTRY				SendTcbFreeList;
	LIST_ENTRY				SendTcbPendingList;
	LIST_ENTRY				SendTcbBusyList;
	LIST_ENTRY				SendTcbcompletionList;
	PUCHAR					TCBMem;
	PUCHAR					TCBData;
	DWORD					TCBUsedCount;
	LONG					nTcbPendingSend;
	LONG					nTcbBusySend;
	LONG					nTcbCompletionSend;
	
	NDIS_SPIN_LOCK			SendLock;
	DWORD					DelayTxComplete;
	NDIS_WORK_ITEM			SendsWaitingWorkItem;
	LONG					IsSendsWaitingWorkItemQueued;
	NDIS_EVENT				SendsWaitingWorkItemReady;
	NDIS_WORK_ITEM			SendsCompletionWorkItem;
	LONG					IsSendsCompletionWorkItemQueued;
	NDIS_EVENT				IsSendsCompletionWorkItemReady;
	BOOLEAN					IsSendReady;
	LONG					MaxTCBs;
	DWORD					TxOnePktperTcb;
	DWORD					TxBatching;
	DWORD					TxBurstMax;

	//
	// Variables to track resources for the Receive operation
	//
	DWORD					StopBulkInOnRxEmpty;
	LIST_ENTRY				BulkInFreeList;
	LIST_ENTRY				BulkInBusyList;
	LIST_ENTRY				BulkInCompletionList;
	LIST_ENTRY				BulkInWaitList;
	LIST_ENTRY				BulkInProcessedList;
	ULONG					BulkInExpectedSeq;
	ULONG					BulkInSubmitedSeq;
	NDIS_SPIN_LOCK			BulkInLock;
	LONG					nBusyRecv;
	LONG					nCompletionRecv;
	LONG					nWaitingRecv;
	LONG					nProcessedRecv;
	NDIS_HANDLE 			RCBPacketPoolHandle;
	NDIS_HANDLE 			RCBBufferPoolHandle;
	PUCHAR					RCBMem;
	PUCHAR					RCBData;
	PRCB					*BulkInCompletionArray;
	NDIS_SPIN_LOCK			RCBProcessLock;

	LIST_ENTRY				RxPktFreeList;
	LIST_ENTRY				RxPktBusyList;
	LONG					nRxPktBusy;
	NDIS_SPIN_LOCK			RxPktLock;
	NDIS_HANDLE 			RxPacketPoolHandle;
	NDIS_HANDLE 			RxBufferPoolHandle;
	PUCHAR					RxData;
	PRCB					PendingRxRCB;
	DWORD					PendingRxPacketSize;
	NDIS_WORK_ITEM			ReadWorkItem;
	LONG					IsReadWorkItemQueued;
	NDIS_EVENT				 ReadWorkItemReady;
   
	
	NDIS_WORK_ITEM			BulkInResumeWorkItem;
	LONG					IsBulkInResumeWorkItemQueued;
	NDIS_EVENT				BulkInResumeWorkItemReady;
	NDIS_WORK_ITEM			BulkInCompletionWorkItem;
	LONG					IsBulkInCompletionWorkItemQueued;
	NDIS_EVENT				BulkInCompletionWorkItemReady;
	BOOLEAN					IsRecvReady;

	
	LONG					MaxRCBs;
	LONG					MaxRxPkts;
	DWORD					RxOnePktperRcb;
	DWORD					RxBurstMax;
	DWORD					RxBurstCap;
	LONG					LowWatermarkRCBs;
	DWORD					MEFBulkInDelay;

	//
	// Variables to track resources for the Interrupt operation
	//
	LIST_ENTRY				IntFreeList;
	LIST_ENTRY				IntBusyList;
	LIST_ENTRY				IntProcessedList;
	LIST_ENTRY				IntCompletionList;
	NDIS_SPIN_LOCK			IntLock;
	LONG					nBusyInt;
	LONG					nProcessedInt;
	LONG					nCompletionInt;
	PUCHAR					ICBMem;
	NDIS_WORK_ITEM			IntStatWorkItem;
	LONG					IsIntStatWorkItemQueued;
	NDIS_WORK_ITEM			PostIntsWorkItem;
	NDIS_WORK_ITEM			IntsCompletionWorkItem;
	LONG					IsIntsCompletionWorkItemQueued;
	NDIS_EVENT				IsIntsCompletionWorkItemReady;
	LONG					IsPostIntsWorkItemQueued;
	LONG					MaxICBs;
	LONG					LowWatermarkICBs;
	BOOLEAN					IsIntReady;

	NDIS_WORK_ITEM			LinkCheckWorkItem;
	LONG					IsLinkCheckWorkItemQueued;

	NDIS_WORK_ITEM			UsbErrorRecoveryWorkItem;
	LONG					IsUsbErrorRecoveryWorkItemQueued;

	//
	// Packet Filter and look ahead size.
	//
	ULONG					PacketFilter;
	ULONG					ulLookAhead;

	// multicast list
	ULONG					ulMCListSize;
	UCHAR					MCList[NIC_MAX_MCAST_LIST][ETH_LENGTH_OF_ADDRESS];

	// Packet counts
	ULONG64 				nPacketsTransmittedSuccessfully;	// GoodTransmits
	ULONG64 				nPacketsIndicatedOnRx;				// GoodReceives

	//Packet Counts (HW)
	ULONG64 				TransmitsGood;
	ULONG64 				ReceivesGood;

	 // Count of transmit errors & other stats.
	ULONG					TxAbortExcessCollisions;
	ULONG					TxLateCollisions;
	ULONG					TxDmaUnderrun;
	ULONG					TxLostCRS;
	ULONG					TxOKButDeferred;
	ULONG					OneRetry;
	ULONG					MoreThanOneRetry;
	ULONG					TotalRetries;
	ULONG					TransmitFailuresOther;
	ULONG					TxPauseFrames;
	ULONG					TxBadFrames;
    ULONG                   EeeTxLpiTransitions;
    ULONG                   EeeTxLpiTime;

	// Count of receive errors
	ULONG					RcvCrcErrors;
	ULONG					RcvAlignmentErrors;
	ULONG					RcvResourceErrors;
	ULONG					RcvDmaOverrunErrors;
	ULONG					RcvCdtFrames;
	ULONG					RcvRuntErrors;
	ULONG					RcvFrameTooLongErrors;
	ULONG					RcvLateCollisionErrors;
	ULONG					RcvBadErrors;
    ULONG                   EeeRxLpiTransitions;
    ULONG                   EeeRxLpiTime;

#ifdef USE_THREAD
	HANDLE					RecvThreadEvent;
	HANDLE					hRecvThread;
	BOOLEAN					RecvThreadExitFlag;
	
	HANDLE					SendThreadEvent;
	HANDLE					hSendThread;
	BOOLEAN					SendThreadExitFlag;
#endif

#ifdef USE_WORK_ITEM_NO_OUT
	NDIS_WORK_ITEM			RecvWorkItem;
	NDIS_EVENT				RecvWorkItemReady;

	NDIS_WORK_ITEM			SendWorkItem;
	NDIS_EVENT				SendWorkItemReady;

	BOOLEAN					SendWorkItemExitFlag;
	BOOLEAN 				RecvWorkItemExitFlag;
#endif

#ifdef USE_INT_PIPE
#ifdef USE_THREAD
	HANDLE					IntThreadEvent;
	HANDLE					hIntThread;
	BOOLEAN					IntThreadExitFlag;
#endif

#ifdef USE_WORK_ITEM_NO_OUT

	NDIS_WORK_ITEM			IntsWorkItem;
	NDIS_EVENT				IntsWorkItemReady;
	BOOLEAN					IntWorkItemExitFlag;
#endif
	
#endif
	//
	// Statistic for debugging & validation purposes
	//
	ULONG					nIntsPosted;
	ULONG					nIntsCompletedWithError;
	ULONG					nIntsCompletedSuccessfully;

	ULONG					nBulkInPosted;
	ULONG					nBulkInCompletedWithError;
	ULONG					nBulkInCompletedSuccessfully;
	ULONG64 				nRxPacketsReturned;
	ULONG64 				nBulkInBytesRead;

	ULONG					nSendPacketsPended;
	ULONG					nSendPacketsArrived;
	ULONG					nSendPacketsCopied;
	ULONG					nSendBytesCopied;
	ULONG	 				nSendPacketsCompleted;
	ULONG					nBulkOutPosted;
	ULONG					nBulkOutCompletedWithError;
	ULONG					nBulkOutCompletedSuccessfully;
	ULONG64 				nBulkOutBytesWritten;

	ULONG					nSendsWaitingWorkItemScheduled;
	ULONG					nSendsCompletionWorkItemScheduled;
	ULONG					nReadWorkItemScheduled;
	ULONG					nBulkInResumeWorkItemScheduled;
	ULONG					nBulkIncompletionWorkItemScheduled;
	ULONG					nIntStatWorkItemScheduled;
	ULONG					nPostIntsWorkItemScheduled;
	ULONG					nLinkCheckWorkItemScheduled;
	ULONG					nUsbErrorRecoveryWorkItemScheduled;
	ULONG					nBulkInCancelledWorkItemScheduled;
	ULONG					nBulkOutCancelledWorkItemScheduled;
	ULONG					nIntsCancelledWorkItemScheduled;
	ULONG					nIntsCompletionWorkItemScheduled;

	DWORD					WakeupGlobalEventCounter;
	DWORD					WakeupWOLEventCounter;
	DWORD					WakeupLinkEventCounter;
	DWORD					WakeupMPEventCounter;

	DWORD					HardwareStatus;

	// HW Stats
	//
	// Make these the last ones since they are pragma pack defined
	// and could cause missalignment of fields following in adapter
	// structure
	//
	LAN9500_RX_STATS		RxStats;
	LAN9500_TX_STATS		TxStats;
	
	DWORD					ResetOnCorruptedRx;
	DWORD					CorruptedRxCount;

} MP_ADAPTER, *PMP_ADAPTER;

typedef struct _MP_GLOBAL_DATA {
	BOOL DeviceAttached;
	BOOL DriverEntryInit;
	DWORD InstancesCount;
	CRITICAL_SECTION	Access_Instance;
	PMP_ADAPTER Adapter[Num_of_Instances_supported];
	int Instances[Num_of_Instances_supported];
	BOOL InstanceRemoving[Num_of_Instances_supported];
} MP_GLOBAL_DATA, *PMP_GLOBAL_DATA;

//extern MP_GLOBAL_DATA GlobalData;

//*****************************************************************************
//
// F U N C T I O N	  P R O T O T Y P E S
//
//*****************************************************************************

BOOL
DllEntry(
   HANDLE hDllHandle,
   DWORD  dwReason,
   LPVOID lpreserved
   );

BOOL
USBInstallDriver(
   LPCWSTR szDriverLibFile
   );

BOOL
USBUnInstallDriver(
   VOID
   );

BOOL
USBDeviceAttach(
   USB_HANDLE		hDevice,
   LPCUSB_FUNCS 	UsbFuncs,
   LPCUSB_INTERFACE UsbInterface,
   LPCWSTR			UniqueDriverId,
   LPBOOL			AcceptControl,
   LPCUSB_DRIVER_SETTINGS UsbDriverSettings,
   DWORD Unused
   );

BOOL WINAPI
UsbDeviceNotify(
   LPVOID lpvNotifyParameter,
   DWORD  dwCode,
   LPDWORD *dwInfo1,
   LPDWORD *dwInfo2,
   LPDWORD *dwInfo3,
   LPDWORD *dwInfo4
   );

VOID 
RemoveDeviceContext(
	PMP_ADAPTER pUsbDevice
	);

DWORD
NdisStartUpThread(
	LPVOID Context
	);

VOID
StopUsbThread(
	PMP_ADAPTER pDevice,
	BOOL bWaitForStartup
	);

LPCUSB_INTERFACE
ParseUsbDescriptors(
   USB_HANDLE		hUsbDevice,
   LPCUSB_FUNCS 	UsbFuncs,
   LPCUSB_INTERFACE CurInterface,
   LPUSHORT 		ConfigIndex,
   PUCHAR			PdmAttributes
   );

BOOL
SetUsbInterface(
   PMP_ADAPTER pUsbDevice
   );

VOID
ClosePipe(
	PMP_ADAPTER pUsbDevice,
	PIPE Pipes
	);

VOID
CloseAllPipes(
	PMP_ADAPTER pUsbDevice
	);

BOOLEAN
NICResetPipe(
	PMP_ADAPTER pUsbDevice,
	PIPE Usbpipe
	);

NIC_STATUS 
NICWriteRegister(
	PMP_ADAPTER  pUsbDevice, 
	USHORT index, 
	PDWORD pbBuffer
	);

NIC_STATUS 
NICReadRegister(
	PMP_ADAPTER  pUsbDevice, 
	USHORT index,
	PDWORD pbBuffer
	);

NIC_STATUS 
NICReadPhy( 
	PMP_ADAPTER  pUsbDevice,
	DWORD Register, 
	DWORD *pValue32
	);

NIC_STATUS
 NICReadPhyAddress(
	PMP_ADAPTER  pUsbDevice,
	DWORD PhyAddress,
	DWORD Register,
	DWORD *pValue32,
	DWORD ReadDWord
	);

NIC_STATUS 
NICWritePhy(
	IN PMP_ADAPTER	pUsbDevice,
	IN DWORD Register,	
	DWORD Value32
	);

NIC_STATUS
NICWritePhyAddress(
	IN PMP_ADAPTER	pUsbDevice,
	DWORD PhyAddress,
	DWORD Register,
	DWORD Value32,
	DWORD ReadDWord
	);

NIC_STATUS
NICReadStats(
	IN PMP_ADAPTER Adapter,
	OUT PUCHAR pStats
	);

NIC_STATUS
NICReadEepromByte(
	IN PMP_ADAPTER Adapter,
	IN WORD Address,
	OUT PBYTE pValue
	) ;

NIC_STATUS
NICWriteEepromByte(
	IN PMP_ADAPTER Adapter,
	IN WORD Address,
	IN PBYTE pValue
	) ;

NIC_STATUS
NICReadEepromBlock(
	IN PMP_ADAPTER Adapter,
	IN WORD Address,
	IN DWORD Length,
	OUT PBYTE pValue
	) ;

NIC_STATUS
NICWriteEepromBlock(
	IN PMP_ADAPTER Adapter,
	IN WORD Address,
	IN DWORD Length,
	IN PBYTE pValue
	) ;

NIC_STATUS
NICEepromCommand(
	IN PMP_ADAPTER Adapter,
	IN DWORD Command
	) ;

NIC_STATUS
NICEepromReload(
	IN PMP_ADAPTER Adapter
	) ;

void GetDriverInfo();

NIC_STATUS
NICSetGpo(
	IN PMP_ADAPTER Adapter,
	IN DWORD Gpo,
	IN DWORD State
	);

//--------------------------------------
// Miniport routines
//--------------------------------------

NDIS_STATUS 
DriverEntry(
	IN PVOID DriverObject,
	IN PVOID RegistryPath
	);

NDIS_STATUS
MPInitialize(
	OUT PNDIS_STATUS OpenErrorStatus,
	OUT PUINT SelectedMediumIndex,
	IN PNDIS_MEDIUM MediumArray,
	IN UINT MediumArraySize,
	IN NDIS_HANDLE MiniportAdapterHandle,
	IN NDIS_HANDLE WrapperConfigurationContext
	);

BOOLEAN
MPCheckForHang(
	IN NDIS_HANDLE MiniportAdapterContext
	);

VOID
MPHalt(
	IN	NDIS_HANDLE MiniportAdapterContext
	);

NDIS_STATUS
MPReset(
	OUT PBOOLEAN AddressingReset,
	IN	NDIS_HANDLE MiniportAdapterContext
	);

VOID
MPShutdown(
	IN	NDIS_HANDLE MiniportAdapterContext
	);

NDIS_STATUS
MPSetInformation(
	IN NDIS_HANDLE									MiniportAdapterContext,
	IN NDIS_OID 									Oid,
	__in_bcount(InformationBufferLength) IN PVOID	InformationBuffer,
	IN ULONG										InformationBufferLength,
	OUT PULONG										BytesRead,
	OUT PULONG										BytesNeeded
	);

NDIS_STATUS
MPQueryInformation(
	IN NDIS_HANDLE MiniportAdapterContext,
	IN NDIS_OID Oid,
	IN PVOID InformationBuffer,
	IN ULONG InformationBufferLength,
	OUT PULONG BytesWritten,
	OUT PULONG BytesNeeded
	);

VOID
MPSendPackets(
	IN	NDIS_HANDLE 			MiniportAdapterContext,
	IN	PPNDIS_PACKET			PacketArray,
	IN	UINT					NumberOfPackets
	);

VOID
MPReturnPacket(
	IN NDIS_HANDLE	MiniportAdapterContext,
	IN PNDIS_PACKET Packet
	);

//--------------------------------------------------
//Init.c funtion prototypes 
//--------------------------------------------------
NDIS_STATUS
NICAllocAdapter(
	PMP_ADAPTER Adapter
	);

VOID
NICFreeAdapter(
	PMP_ADAPTER Adapter
	);

void
USBDeviceOperationalSpeed(
	IN PMP_ADAPTER		Adapter
	);

NDIS_STATUS
NICReadRegParameters(
	PMP_ADAPTER Adapter,
	NDIS_HANDLE ConfigurationHandle
	);

NDIS_STATUS
NICInitializeAdapter(
	IN PMP_ADAPTER Adapter
	);

NTSTATUS
NICInitAdapterWorker(
	PMP_ADAPTER Adapter
	) ;

NDIS_STATUS
NICAllocSendResources(
	PMP_ADAPTER Adapter
	);

VOID
NICFreeSendResources(
	PMP_ADAPTER Adapter
	);

#ifdef USE_INT_PIPE
NDIS_STATUS
NICAllocIntResources(
	PMP_ADAPTER Adapter
	);

VOID
NICFreeIntResources(
	PMP_ADAPTER Adapter
	);
#endif

NDIS_STATUS
NICAllocRecvResources(
	PMP_ADAPTER Adapter
	);

VOID
NICFreeRecvResources(
	PMP_ADAPTER Adapter
	);

NDIS_STATUS
NICAllocWolResources(
	PMP_ADAPTER Adapter
	);

VOID
NICFreeWolResources(
	PMP_ADAPTER Adapter
	);

NIC_STATUS
NICResetAdapter(
	IN PMP_ADAPTER Adapter
	) ;

BOOLEAN
NICIsCardPresent(
	IN PMP_ADAPTER Adapter
	) ;

NIC_STATUS
NICGetPermanentNodeAddress(
	IN PMP_ADAPTER Adapter
	) ;

void
NICGetWOLPatternCount(
	IN PMP_ADAPTER Adapter
	) ;

VOID
StartLinkCheck(
	IN PMP_ADAPTER Adapter
	);

NDIS_STATUS
SafeCancelTimer(
	IN PMP_ADAPTER Adapter,
	IN PNDIS_SPIN_LOCK Spinlock,
	IN PNDIS_MINIPORT_TIMER Timer,
	IN PNDIS_EVENT Event,
	IN UINT Timeout
	);

NDIS_STATUS SafeCancelLinkCheck(
	IN PMP_ADAPTER Adapter
	);

NIC_STATUS
NICSetPHYInterruptCapability(
	IN PMP_ADAPTER Adapter,
   WORD IntMask
	) ;

NIC_STATUS
NICIndicateMediaStatus(
	IN PMP_ADAPTER Adapter
	) ;

NIC_STATUS
NICSetReceiveMask(
	IN PMP_ADAPTER Adapter,
	DWORD ReceiveMask
	) ;

NIC_STATUS
NICInitializeMedia(
	IN PMP_ADAPTER Adapter
	);

NIC_STATUS
NICSetMediaType(
	IN PMP_ADAPTER Adapter
	) ;

BOOL 
NICIsNWAYComplete(
	IN PMP_ADAPTER Adapter
	) ;

void
NICCompleteOidQueryIfPending(
	IN PMP_ADAPTER Adapter
	) ;

NIC_STATUS
NICConfigureWakeEvents(
	IN PMP_ADAPTER Adapter
	) ;

VOID
NICUsbErrorRecoveryWorkItemCallBack(
	PNDIS_WORK_ITEM WorkItem,
	PVOID Context
	) ;

NIC_STATUS
NICReset(
	PMP_ADAPTER Adapter
	);

DWORD
USB_ResetEvent(
	HANDLE hEvent
	);

BOOL
USB_ResetAllPipes(
	PMP_ADAPTER pUsbDevice
	);

DWORD
NICCalculateHash(
	BYTE *pBuffer,
	WORD wLength
	) ;

NIC_STATUS
NICSetHwMulticastTable(
	PMP_ADAPTER Adapter,
	DWORD	NumOfAdresses,
	BYTE	*Addresses
	) ;

NIC_STATUS 
NICSetPowerState(
	PMP_ADAPTER Adapter,
	BYTE	PowerState
	);

BOOLEAN
NICSupportsMEF(
	PMP_ADAPTER Adapter
	);

NIC_STATUS
EepRamRead(
    IN PMP_ADAPTER Adapter,
    DWORD Addr,
    PDWORD pData
    ) ;

NIC_STATUS
EepRamWrite(
    IN PMP_ADAPTER Adapter,
    DWORD Addr,
    DWORD Data
    ) ;

NIC_STATUS 
NICGetPermanentNodeAddressFromEepRAM(
    IN PMP_ADAPTER  Adapter,    
    PDWORD          pdwAddrl,
    PDWORD          pdwAddrh
    ) ;

//---------------------------------------------------
//send.c funtion prototype
//---------------------------------------------------

NIC_STATUS
NICStartTxPath(
	PMP_ADAPTER Adapter
	);

VOID
NICPostSendsWaitingWorkItemCallBack(
	PNDIS_WORK_ITEM WorkItem,
	PVOID Context
	);

VOID
MPSend(
	PMP_ADAPTER 	Adapter
	);


NTSTATUS
NICPostWriteRequest(
	PMP_ADAPTER Adapter,
	PTCB	pTCB
	);

NIC_STATUS
NICStopAndFlushTxPath(
	PMP_ADAPTER Adapter
	);

NIC_STATUS
NICStopTxPath(
	PMP_ADAPTER Adapter
	);

NIC_STATUS
NICStopBulkOut(
	PMP_ADAPTER Adapter
	);


VOID
NICFreeSendTCB(
	IN PMP_ADAPTER Adapter,
	IN PTCB pTCB
	);

VOID 
NICFreeCompletionTCB(
	IN PMP_ADAPTER Adapter,
	IN PTCB pTCB
	);

VOID
NICFreeQueuedSendPackets(
	PMP_ADAPTER Adapter
	);

BOOLEAN
NICCopyPacket(
	PMP_ADAPTER Adapter,
	PTCB pTCB,
	PNDIS_PACKET Packet
	);

NTSTATUS
NICProcessTCB(
	PMP_ADAPTER Adapter,
	PTCB	pTCB
	);

NTSTATUS
NICWriteRequestCompletion(
	PTCB			pTCB
	);

NTSTATUS
SendsCompletionWorkItemCallBack(
	PNDIS_WORK_ITEM WorkItem, 
	PVOID Context
	);

VOID
NICFreePendingSendPackets(
	PMP_ADAPTER Adapter
	);

//---------------------------------------------------------------
//recieve.c prototype
//---------------------------------------------------------------
VOID
NICPostReadsWorkItemCallBack(
	PNDIS_WORK_ITEM WorkItem,
	PVOID Context
	);

NIC_STATUS
NICStartRxPath(
	PMP_ADAPTER Adapter,
	BOOLEAN NoBulkInStart
	);

VOID
NICStartBulkIn(
	PMP_ADAPTER Adapter
	);

VOID NICPostReads(
	PMP_ADAPTER Adapter
	);

NTSTATUS
NICPostReadRequest(
	PMP_ADAPTER Adapter,
	PRCB	pRCB
	);

VOID
NICProcessBulkInWaitList(
	PMP_ADAPTER Adapter
	);

VOID
NICBulkInResumeWorkItemCallBack(
	PNDIS_WORK_ITEM WorkItem,
	PVOID Context
	);

NTSTATUS
NICReadRequestCompletion(
	PRCB			pRCB
	);

NTSTATUS
BulkInCompletionWorkItemCallBack(
	PNDIS_WORK_ITEM WorkItem, 
	PVOID Context
	);

BOOLEAN
NICProcessByteStreamReceivePackets(
	IN PRCB 			pRCB,
	IN ULONG			BytesInUrb
	);

BOOLEAN
NICProcessMultipleReceivePackets(
	IN PRCB 			pRCB,
	IN ULONG			BytesInUrb
	);

VOID
NICProcessSingleReceivePacket(
	IN PRCB 			pRCB
	);

VOID
NICIndicateReceiveSplitPacket(
	IN PRCB 			pRCB,
	PULONG				pPacketLength
	);

NIC_STATUS
NICStopAndFlushRxPath(
	PMP_ADAPTER Adapter,
	BOOLEAN NoBulkInStop
	);

NIC_STATUS
NICStopRxPath(
	PMP_ADAPTER Adapter,
	BOOLEAN NoBulkInStop
	);

NIC_STATUS
NICStopBulkIn(
	PMP_ADAPTER Adapter
	);

VOID
NICCollectSplitRxPackets(
	PMP_ADAPTER Adapter);

VOID
NICFreeBusyRecvPackets(
	PMP_ADAPTER Adapter
	);

VOID
NICFreeRCB(
	IN PRCB pRCB
	);

BOOLEAN
NICIsPacketAcceptable(
	IN PMP_ADAPTER Adapter,
	IN PUCHAR	pDstMac
	);

BOOLEAN
NICIsMulticastMatch(
	IN PMP_ADAPTER					Adapter,
	IN PUCHAR						pDstMac
	);

//----------------------------------------------------
//Interrupt.c funtion prototype
//----------------------------------------------------
#ifdef USE_INT_PIPE
VOID
NICPostIntsWorkItemCallBack(
	PNDIS_WORK_ITEM WorkItem,
	PVOID Context
	);


NIC_STATUS
NICStartInterruptPipe(
	PMP_ADAPTER Adapter
	);


VOID
NICPostInts(
	PMP_ADAPTER Adapter
	);


NIC_STATUS
NICStopInterruptPipe(
	PMP_ADAPTER Adapter
	);

VOID
NICIntStatWorkItemCallBack(
	PNDIS_WORK_ITEM WorkItem,
	PVOID Context
	);


NTSTATUS
NICPostIntRequest(
	PMP_ADAPTER Adapter,
	PICB	pICB
	);

DWORD
NICIntRequestCompletion(
	PICB	pICB
	);

NTSTATUS
IntsCompletionWorkItemCallBack(
	PNDIS_WORK_ITEM WorkItem, 
	PVOID Context
	);

VOID
NICFreeICB(
	IN PICB pICB
	);

#endif

VOID
NICLinkCheck(
	IN PVOID  SystemSpecific1,
	IN PVOID  FunctionContext,
	IN PVOID  SystemSpecific2,
	IN PVOID  SystemSpecific3
	) ;

int Nic_phy_boost(
    PMP_ADAPTER Adapter, 
    int mode
    );

VOID
NICLinkCheckWorkItemCallBack(
	PNDIS_WORK_ITEM WorkItem,
	PVOID Context
	) ;

NIC_STATUS
NICServicePhyInterrupt(
	PMP_ADAPTER Adapter
	) ;

NIC_STATUS
NICCheckOtherPhysLinkStatus(
	PMP_ADAPTER Adapter,
	int *OtherPhysLinkStatus
	);
//-----------------------------------------------
//request.c funtion prototype
//-----------------------------------------------

ULONG
NICGetMediaConnectStatus(
	PMP_ADAPTER Adapter
	);

ULONG
NICGetLinkSpeed(
	PMP_ADAPTER Adapter
	);

NDIS_STATUS
NICSetPacketFilter(
	IN PMP_ADAPTER Adapter,
	IN ULONG PacketFilter
	);

NDIS_STATUS
NICSetMulticastList(
	IN PMP_ADAPTER									Adapter,
	__in_bcount(InformationBufferLength) IN PVOID	InformationBuffer,
	IN ULONG										InformationBufferLength,
	OUT PULONG										pBytesRead,
	OUT PULONG										pBytesNeeded
	);

NDIS_STATUS
NICScheduleRequest(
	IN PMP_ADAPTER					Adapter,
	IN NDIS_REQUEST_TYPE			RequestType,
	IN NDIS_OID 					Oid,
	IN PVOID						InformationBuffer,
	IN ULONG						InformationBufferLength,
	OUT PULONG						BytesReadOrWritten,
	OUT PULONG						BytesNeeded
	);

VOID
NICRequestWorkItemCallback(
	IN PNDIS_WORK_ITEM WorkItem,
	IN PVOID Context
	) ;

BYTE
NICGetWOLPattern(
	IN PMP_ADAPTER Adapter
	) ;

NDIS_STATUS
NICAddWOLSample(
	IN PMP_ADAPTER Adapter,
	PLM_WOL_SAMPLE_STRUCT pLMWolSample
	) ;

PLM_WOL_SAMPLE_STRUCT
NICFindWolSample(
	IN PMP_ADAPTER Adapter,
	PLM_WOL_SAMPLE_STRUCT pLMWolSample
	) ;

void
NICRemoveWOLSample(
	IN PMP_ADAPTER Adapter,
	PLM_WOL_SAMPLE_STRUCT pLMWolSample
	) ;

WORD
NicCalculateCrc16(
	const BYTE * bpData,
	const DWORD dwLen,
	const DWORD fBitReverse
	) ;

PUCHAR
DbgGetOidName(
	ULONG oid
	);

//-----------------------------------------------
//Ioctl.c funtion prototype
//-----------------------------------------------

NIC_STATUS DumpLanRegs(PMP_ADAPTER Adapter);
NIC_STATUS DumpMacRegs(PMP_ADAPTER Adapter);
NIC_STATUS DumpPhyRegs(PMP_ADAPTER Adapter);
void DumpRXStatus (PMP_ADAPTER Adapter);
void DumpTXStatus (PMP_ADAPTER Adapter);
NIC_STATUS WriteFileToE2P(PMP_ADAPTER Adapter) ;
BOOL GetE2pSize (PMP_ADAPTER Adapter, int *E2pSize);

VOID
 NICPostReadsCallBack(
	PMP_ADAPTER 	Adapter
	);

VOID
NICPostSendsWaitingCallBack(
	PMP_ADAPTER 	Adapter
	);

NTSTATUS
BulkInCompletionCallBack(
	PMP_ADAPTER Adapter
	);

NTSTATUS
SendsCompletionCallBack(
	PMP_ADAPTER 	Adapter
	);

NTSTATUS
IntsCompletionCallBack(
   PMP_ADAPTER Adapter
	);

#ifdef USE_THREAD
DWORD
RecvThread(
	LPVOID Context
	);

DWORD
SendThread(
	LPVOID Context
	);
#endif

#ifdef USE_WORK_ITEM_NO_OUT

DWORD RecvWorkItemCallBack(
		PNDIS_WORK_ITEM WorkItem, 
		PVOID Context
		);

DWORD SendWorkItemCallBack(
		PNDIS_WORK_ITEM WorkItem, 
		PVOID Context
		);
#endif

#ifdef USE_INT_PIPE
#ifdef USE_THREAD

DWORD IntThread(
		LPVOID Context
		);
#endif
#ifdef USE_WORK_ITEM_NO_OUT
DWORD IntWorkItemCallback(
		PNDIS_WORK_ITEM WorkItem, 
		PVOID Context
		);
#endif
#endif

#endif
